# coding: utf8
from __future__ import unicode_literals

from ...symbols import POS, ADJ, NUM, DET, ADV, ADP, X, VERB, NOUN
from ...symbols import PROPN, PART, INTJ, PRON, SCONJ, AUX, CCONJ

TAG_MAP = {
    "ADJ_Abbr=Yes": {POS: ADJ, "Abbr": "Yes"},
    "ADJ_Degree=Pos|NumForm=Word|NumType=Ord": {
        POS: ADJ,
        "Degree": "Pos",
        "NumForm": "Word",
        "NumType": "Ord",
    },
    "ADJ_Degree=Pos": {POS: ADJ, "Degree": "Pos"},
    "ADJ_Degree=Pos|Style=Coll": {POS: ADJ, "Degree": "Pos", "Style": "Coll"},
    "ADJ_Degree=Pos|Style=Expr": {POS: ADJ, "Degree": "Pos", "Style": "Expr"},
    "ADJ_Degree=Sup": {POS: ADJ, "Degree": "Sup"},
    "ADJ_NumForm=Digit|NumType=Ord": {POS: ADJ, "NumForm": "Digit", "NumType": "Ord"},
    "ADJ_NumForm=Word|NumType=Card": {POS: ADJ, "NumForm": "Word", "NumType": "Card"},
    "ADJ_NumForm=Word|NumType=Ord": {POS: ADJ, "NumForm": "Word", "NumType": "Ord"},
    "ADJ_Style=Coll": {POS: ADJ, "Style": "Coll"},
    "ADJ_Style=Expr": {POS: ADJ, "Style": "Expr"},
    "ADP_AdpType=Post|Case=Dat": {POS: ADP, "AdpType": "Post", "Case": "Dat"},
    "ADP_AdpType=Post|Case=Nom": {POS: ADP, "AdpType": "Post", "Case": "Nom"},
    "ADP_AdpType=Post|Number=Plur|Person=3": {
        POS: ADP,
        "AdpType": "Post",
        "Number": "Plur",
        "Person": "3",
    },
    "ADP_AdpType=Post": {POS: ADP, "AdpType": "Post"},
    "ADP_AdpType=Prep": {POS: ADP, "AdpType": "Prep"},
    "ADP_AdpType=Prep|Style=Arch": {POS: ADP, "AdpType": "Prep", "Style": "Arch"},
    "ADV_Degree=Cmp": {POS: ADV, "Degree": "Cmp"},
    "ADV_Degree=Pos": {POS: ADV, "Degree": "Pos"},
    "ADV_Degree=Sup": {POS: ADV, "Degree": "Sup"},
    "ADV_Distance=Dist|PronType=Dem": {POS: ADV, "Distance": "Dist", "PronType": "Dem"},
    "ADV_Distance=Dist|PronType=Exc": {POS: ADV, "Distance": "Dist", "PronType": "Exc"},
    "ADV_Distance=Med|PronType=Dem": {POS: ADV, "Distance": "Med", "PronType": "Dem"},
    "ADV_Distance=Med|PronType=Dem|Style=Coll": {
        POS: ADV,
        "Distance": "Med",
        "PronType": "Dem",
        "Style": "Coll",
    },
    "ADV_NumForm=Word|NumType=Card|PronType=Tot": {
        POS: ADV,
        "NumForm": "Word",
        "NumType": "Card",
        "PronType": "Tot",
    },
    "ADV_PronType=Dem": {POS: ADV, "PronType": "Dem"},
    "ADV_PronType=Exc": {POS: ADV, "PronType": "Exc"},
    "ADV_PronType=Ind": {POS: ADV, "PronType": "Ind"},
    "ADV_PronType=Int": {POS: ADV, "PronType": "Int"},
    "ADV_PronType=Int|Style=Coll": {POS: ADV, "PronType": "Int", "Style": "Coll"},
    "ADV_PronType=Rel": {POS: ADV, "PronType": "Rel"},
    "ADV_Style=Coll": {POS: ADV, "Style": "Coll"},
    "ADV_Style=Rare": {POS: ADV, "Style": "Rare"},
    "AUX_Aspect=Imp|Mood=Ind|Number=Plur|Person=1|Polarity=Neg|Tense=Pres|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "1",
        "Polarity": "Neg",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Plur|Person=2|Polarity=Pos|Tense=Pres|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "2",
        "Polarity": "Pos",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Plur|Person=3|Polarity=Neg|Tense=Imp|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Neg",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Plur|Person=3|Polarity=Neg|Tense=Pres|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Neg",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Plur|Person=3|Polarity=Pos|Tense=Imp|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Pos",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Plur|Person=3|Polarity=Pos|Tense=Pres|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Pos",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Polarity=Neg|Tense=Imp|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Neg",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Polarity=Neg|Tense=Pres|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Neg",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Polarity=Pos|Tense=Imp|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Pos",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Polarity=Pos|Tense=Pres|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Pos",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Sing|Person=2|Polarity=Neg|Tense=Pres|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "2",
        "Polarity": "Neg",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Sing|Person=2|Polarity=Pos|Tense=Pres|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "2",
        "Polarity": "Pos",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Polarity=Neg|Tense=Imp|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Neg",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Polarity=Neg|Tense=Pres|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Neg",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Polarity=Pos|Tense=Imp|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Tense": "Imp",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Polarity=Pos|Tense=Pres|VerbForm=Fin": {
        POS: AUX,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Tense": "Pres",
        "VerbForm": "Fin",
    },
    "AUX_Aspect=Imp|VerbForm=Part": {POS: AUX, "Aspect": "Imp", "VerbForm": "Part"},
    "AUX_Aspect=Perf|VerbForm=Part": {POS: AUX, "Aspect": "Perf", "VerbForm": "Part"},
    "AUX_Aspect=Prosp|VerbForm=Part": {POS: AUX, "Aspect": "Prosp", "VerbForm": "Part"},
    "AUX_Polarity=Pos": {POS: AUX, "Polarity": "Pos"},
    "CCONJ_ConjType=Comp": {POS: CCONJ, "ConjType": "Comp"},
    "CCONJ_ConjType=Comp|Style=Coll": {POS: CCONJ, "ConjType": "Comp", "Style": "Coll"},
    "DET_Case=Gen|Distance=Med|Number=Plur|Poss=Yes|PronType=Dem": {
        POS: DET,
        "Case": "Gen",
        "Distance": "Med",
        "Number": "Plur",
        "Poss": "Yes",
        "PronType": "Dem",
    },
    "DET_Case=Gen|Distance=Med|Number=Sing|Poss=Yes|PronType=Dem": {
        POS: DET,
        "Case": "Gen",
        "Distance": "Med",
        "Number": "Sing",
        "Poss": "Yes",
        "PronType": "Dem",
    },
    "DET_Case=Gen|Number=Plur|Person=1|Poss=Yes|PronType=Prs": {
        POS: DET,
        "Case": "Gen",
        "Number": "Plur",
        "Person": "1",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "DET_Case=Gen|Number=Plur|Person=2|Polite=Infm|Poss=Yes|PronType=Prs": {
        POS: DET,
        "Case": "Gen",
        "Number": "Plur",
        "Person": "2",
        "Polite": "Infm",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "DET_Case=Gen|Number=Plur|Person=3|Poss=Yes|PronType=Emp": {
        POS: DET,
        "Case": "Gen",
        "Number": "Plur",
        "Person": "3",
        "Poss": "Yes",
        "PronType": "Emp",
    },
    "DET_Case=Gen|Number=Plur|Person=3|Poss=Yes|PronType=Emp|Reflex=Yes": {
        POS: DET,
        "Case": "Gen",
        "Number": "Plur",
        "Person": "3",
        "Poss": "Yes",
        "PronType": "Emp",
        "Reflex": "Yes",
    },
    "DET_Case=Gen|Number=Sing|Person=1|Poss=Yes|PronType=Prs": {
        POS: DET,
        "Case": "Gen",
        "Number": "Sing",
        "Person": "1",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "DET_Case=Gen|Number=Sing|Person=2|Polite=Infm|Poss=Yes|PronType=Prs": {
        POS: DET,
        "Case": "Gen",
        "Number": "Sing",
        "Person": "2",
        "Polite": "Infm",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "DET_Case=Gen|Number=Sing|Person=3|Poss=Yes|PronType=Emp": {
        POS: DET,
        "Case": "Gen",
        "Number": "Sing",
        "Person": "3",
        "Poss": "Yes",
        "PronType": "Emp",
    },
    "DET_Case=Gen|Number=Sing|Person=3|Poss=Yes|PronType=Emp|Reflex=Yes": {
        POS: DET,
        "Case": "Gen",
        "Number": "Sing",
        "Person": "3",
        "Poss": "Yes",
        "PronType": "Emp",
        "Reflex": "Yes",
    },
    "DET_Case=Gen|Number=Sing|Person=3|Poss=Yes|PronType=Prs": {
        POS: DET,
        "Case": "Gen",
        "Number": "Sing",
        "Person": "3",
        "Poss": "Yes",
        "PronType": "Prs",
    },
    "DET_Case=Gen|Number=Sing|Poss=Yes|PronType=Rel": {
        POS: DET,
        "Case": "Gen",
        "Number": "Sing",
        "Poss": "Yes",
        "PronType": "Rel",
    },
    "DET_Distance=Dist|PronType=Dem": {POS: DET, "Distance": "Dist", "PronType": "Dem"},
    "DET_Distance=Dist|PronType=Dem|Style=Coll": {
        POS: DET,
        "Distance": "Dist",
        "PronType": "Dem",
        "Style": "Coll",
    },
    "DET_Distance=Dist|PronType=Dem|Style=Vrnc": {
        POS: DET,
        "Distance": "Dist",
        "PronType": "Dem",
        "Style": "Vrnc",
    },
    "DET_Distance=Med|PronType=Dem": {POS: DET, "Distance": "Med", "PronType": "Dem"},
    "DET_Distance=Med|PronType=Dem|Style=Coll": {
        POS: DET,
        "Distance": "Med",
        "PronType": "Dem",
        "Style": "Coll",
    },
    "DET_Distance=Prox|PronType=Dem": {POS: DET, "Distance": "Prox", "PronType": "Dem"},
    "DET_Distance=Prox|PronType=Dem|Style=Coll": {
        POS: DET,
        "Distance": "Prox",
        "PronType": "Dem",
        "Style": "Coll",
    },
    "DET_PronType=Art": {POS: DET, "PronType": "Art"},
    "DET_PronType=Exc": {POS: DET, "PronType": "Exc"},
    "DET_PronType=Ind": {POS: DET, "PronType": "Ind"},
    "DET_PronType=Int": {POS: DET, "PronType": "Int"},
    "DET_PronType=Tot": {POS: DET, "PronType": "Tot"},
    "DET_PronType=Tot|Style=Arch": {POS: DET, "PronType": "Tot", "Style": "Arch"},
    "INTJ_Style=Vrnc": {POS: INTJ, "Style": "Vrnc"},
    "NOUN_Abbr=Yes|Animacy=Nhum|Case=Dat|Definite=Ind|Number=Plur": {
        POS: NOUN,
        "Abbr": "Yes",
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Plur",
    },
    "NOUN_Abbr=Yes|Animacy=Nhum|Case=Nom|Definite=Ind|Number=Sing": {
        POS: NOUN,
        "Abbr": "Yes",
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Sing",
    },
    "NOUN_Animacy=Hum|Case=Abl|Definite=Ind|Number=Plur": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Abl",
        "Definite": "Ind",
        "Number": "Plur",
    },
    "NOUN_Animacy=Hum|Case=Abl|Definite=Ind|Number=Plur|Style=Slng": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Abl",
        "Definite": "Ind",
        "Number": "Plur",
        "Style": "Slng",
    },
    "NOUN_Animacy=Hum|Case=Abl|Definite=Ind|Number=Sing": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Abl",
        "Definite": "Ind",
        "Number": "Sing",
    },
    "NOUN_Animacy=Hum|Case=Dat|Definite=Def|Number=Plur": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Definite": "Def",
        "Number": "Plur",
    },
    "NOUN_Animacy=Hum|Case=Dat|Definite=Def|Number=Sing": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Definite": "Def",
        "Number": "Sing",
    },
    "NOUN_Animacy=Hum|Case=Dat|Definite=Def|Number=Sing|Style=Slng": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Definite": "Def",
        "Number": "Sing",
        "Style": "Slng",
    },
    "NOUN_Animacy=Hum|Case=Dat|Definite=Ind|Number=Assoc": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Assoc",
    },
    "NOUN_Animacy=Hum|Case=Dat|Definite=Ind|Number=Plur": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Plur",
    },
    "NOUN_Animacy=Hum|Case=Dat|Definite=Ind|Number=Plur|Style=Coll": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Plur",
        "Style": "Coll",
    },
    "NOUN_Animacy=Hum|Case=Dat|Definite=Ind|Number=Plur|Style=Slng": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Plur",
        "Style": "Slng",
    },
    "NOUN_Animacy=Hum|Case=Dat|Definite=Ind|Number=Sing": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Sing",
    },
    "NOUN_Animacy=Hum|Case=Dat|Definite=Ind|Number=Sing|Style=Arch": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Sing",
        "Style": "Arch",
    },
    "NOUN_Animacy=Hum|Case=Dat|Number=Sing|Number=Sing|Person=1": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Number": "Sing",
        "Number": "Sing",
        "Person": "1",
    },
    "NOUN_Animacy=Hum|Case=Dat|Number=Sing|Number=Sing|Person=1|Style=Coll": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Number": "Sing",
        "Number": "Sing",
        "Person": "1",
        "Style": "Coll",
    },
    "NOUN_Animacy=Hum|Case=Ins|Definite=Ind|Number=Sing": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Ins",
        "Definite": "Ind",
        "Number": "Sing",
    },
    "NOUN_Animacy=Hum|Case=Nom|Definite=Def|Number=Plur": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Def",
        "Number": "Plur",
    },
    "NOUN_Animacy=Hum|Case=Nom|Definite=Def|Number=Plur|Style=Slng": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Def",
        "Number": "Plur",
        "Style": "Slng",
    },
    "NOUN_Animacy=Hum|Case=Nom|Definite=Def|Number=Sing": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Def",
        "Number": "Sing",
    },
    "NOUN_Animacy=Hum|Case=Nom|Definite=Def|Number=Sing|Style=Coll": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Def",
        "Number": "Sing",
        "Style": "Coll",
    },
    "NOUN_Animacy=Hum|Case=Nom|Definite=Ind|Number=Assoc": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Assoc",
    },
    "NOUN_Animacy=Hum|Case=Nom|Definite=Ind|Number=Plur": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Plur",
    },
    "NOUN_Animacy=Hum|Case=Nom|Definite=Ind|Number=Plur|Style=Coll": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Plur",
        "Style": "Coll",
    },
    "NOUN_Animacy=Hum|Case=Nom|Definite=Ind|Number=Plur|Style=Slng": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Plur",
        "Style": "Slng",
    },
    "NOUN_Animacy=Hum|Case=Nom|Definite=Ind|Number=Plur|Typo=Yes": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Plur",
        "Typo": "Yes",
    },
    "NOUN_Animacy=Hum|Case=Nom|Definite=Ind|Number=Sing": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Sing",
    },
    "NOUN_Animacy=Hum|Case=Nom|Definite=Ind|Number=Sing|Style=Coll": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Sing",
        "Style": "Coll",
    },
    "NOUN_Animacy=Hum|Case=Nom|Number=Sing|Number=Sing|Person=1": {
        POS: NOUN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Number": "Sing",
        "Number": "Sing",
        "Person": "1",
    },
    "NOUN_Animacy=Nhum|Case=Abl|Definite=Ind|Number=Coll": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Abl",
        "Definite": "Ind",
        "Number": "Coll",
    },
    "NOUN_Animacy=Nhum|Case=Abl|Definite=Ind|Number=Plur": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Abl",
        "Definite": "Ind",
        "Number": "Plur",
    },
    "NOUN_Animacy=Nhum|Case=Abl|Definite=Ind|Number=Sing": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Abl",
        "Definite": "Ind",
        "Number": "Sing",
    },
    "NOUN_Animacy=Nhum|Case=Abl|Definite=Ind|Number=Sing|Style=Arch": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Abl",
        "Definite": "Ind",
        "Number": "Sing",
        "Style": "Arch",
    },
    "NOUN_Animacy=Nhum|Case=Abl|Number=Sing|Number=Sing|Person=2": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Abl",
        "Number": "Sing",
        "Number": "Sing",
        "Person": "2",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Def|Number=Coll": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Def",
        "Number": "Coll",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Def|Number=Plur": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Def",
        "Number": "Plur",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Def|Number=Sing|NumForm=Digit": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Def",
        "Number": "Sing",
        "NumForm": "Digit",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Def|Number=Sing|NumForm=Word": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Def",
        "Number": "Sing",
        "NumForm": "Word",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Def|Number=Sing": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Def",
        "Number": "Sing",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Def|Number=Sing|Style=Rare": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Def",
        "Number": "Sing",
        "Style": "Rare",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Def|Number=Sing|Style=Vrnc": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Def",
        "Number": "Sing",
        "Style": "Vrnc",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Ind|Number=Coll": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Coll",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Ind|Number=Plur": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Plur",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Ind|Number=Sing|NumForm=Digit": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Sing",
        "NumForm": "Digit",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Ind|Number=Sing": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Sing",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Ind|Number=Sing|Style=Coll": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Sing",
        "Style": "Coll",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Definite=Ind|Number=Sing|Style=Vrnc": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Sing",
        "Style": "Vrnc",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Number=Coll|Number=Sing|Person=1": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        # "Number": "Coll",
        "Number": "Sing",
        "Person": "1",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Number=Sing|Number=Sing|Person=1": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Number": "Sing",
        "Number": "Sing",
        "Person": "1",
    },
    "NOUN_Animacy=Nhum|Case=Dat|Number=Sing|Number=Sing|Person=2": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Number": "Sing",
        "Number": "Sing",
        "Person": "2",
    },
    "NOUN_Animacy=Nhum|Case=Gen|Definite=Ind|Number=Sing|Style=Arch": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Gen",
        "Definite": "Ind",
        "Number": "Sing",
        "Style": "Arch",
    },
    "NOUN_Animacy=Nhum|Case=Ins|Definite=Ind|Number=Coll": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Ins",
        "Definite": "Ind",
        "Number": "Coll",
    },
    "NOUN_Animacy=Nhum|Case=Ins|Definite=Ind|Number=Plur": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Ins",
        "Definite": "Ind",
        "Number": "Plur",
    },
    "NOUN_Animacy=Nhum|Case=Ins|Definite=Ind|Number=Sing": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Ins",
        "Definite": "Ind",
        "Number": "Sing",
    },
    "NOUN_Animacy=Nhum|Case=Ins|Definite=Ind|Number=Sing|Style=Coll": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Ins",
        "Definite": "Ind",
        "Number": "Sing",
        "Style": "Coll",
    },
    "NOUN_Animacy=Nhum|Case=Ins|Number=Sing|Number=Sing|Person=1": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Ins",
        "Number": "Sing",
        "Number": "Sing",
        "Person": "1",
    },
    "NOUN_Animacy=Nhum|Case=Loc|Definite=Ind|Number=Plur": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Loc",
        "Definite": "Ind",
        "Number": "Plur",
    },
    "NOUN_Animacy=Nhum|Case=Loc|Definite=Ind|Number=Sing": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Loc",
        "Definite": "Ind",
        "Number": "Sing",
    },
    "NOUN_Animacy=Nhum|Case=Loc|Number=Sing|Number=Sing|Person=2": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Loc",
        "Number": "Sing",
        "Number": "Sing",
        "Person": "2",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Definite=Def|Number=Coll": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Def",
        "Number": "Coll",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Definite=Def|Number=Plur|Number=Sing|Poss=Yes": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Def",
        # "Number": "Plur",
        "Number": "Sing",
        "Poss": "Yes",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Definite=Def|Number=Plur": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Def",
        "Number": "Plur",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Definite=Def|Number=Sing|NumForm=Digit": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Def",
        "Number": "Sing",
        "NumForm": "Digit",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Definite=Def|Number=Sing": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Def",
        "Number": "Sing",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Definite=Ind|Number=Coll": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Coll",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Definite=Ind|Number=Coll|Typo=Yes": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Coll",
        "Typo": "Yes",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Definite=Ind|Number=Plur": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Plur",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Definite=Ind|Number=Sing": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Sing",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Definite=Ind": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Ind",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Number=Plur|Number=Sing|Person=2": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        # "Number": "Plur",
        "Number": "Sing",
        "Person": "2",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Number=Sing|Number=Sing|Person=1": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Number": "Sing",
        "Number": "Sing",
        "Person": "1",
    },
    "NOUN_Animacy=Nhum|Case=Nom|Number=Sing|Number=Sing|Person=2": {
        POS: NOUN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Number": "Sing",
        "Number": "Sing",
        "Person": "2",
    },
    "NUM_NumForm=Digit|NumType=Card": {POS: NUM, "NumForm": "Digit", "NumType": "Card"},
    "NUM_NumForm=Digit|NumType=Frac|Typo=Yes": {
        POS: NUM,
        "NumForm": "Digit",
        "NumType": "Frac",
        "Typo": "Yes",
    },
    "NUM_NumForm=Digit|NumType=Range": {
        POS: NUM,
        "NumForm": "Digit",
        "NumType": "Range",
    },
    "NUM_NumForm=Word|NumType=Card": {POS: NUM, "NumForm": "Word", "NumType": "Card"},
    "NUM_NumForm=Word|NumType=Dist": {POS: NUM, "NumForm": "Word", "NumType": "Dist"},
    "NUM_NumForm=Word|NumType=Range": {POS: NUM, "NumForm": "Word", "NumType": "Range"},
    "PART_Polarity=Neg": {POS: PART, "Polarity": "Neg"},
    "PRON_Case=Abl|Definite=Ind|Number=Sing|Person=3|PronType=Prs": {
        POS: PRON,
        "Case": "Abl",
        "Definite": "Ind",
        "Number": "Sing",
        "Person": "3",
        "PronType": "Prs",
    },
    "PRON_Case=Abl|Number=Plur|Person=3|PronType=Prs": {
        POS: PRON,
        "Case": "Abl",
        "Number": "Plur",
        "Person": "3",
        "PronType": "Prs",
    },
    "PRON_Case=Abl|Number=Sing|Person=2|Polite=Infm|PronType=Prs": {
        POS: PRON,
        "Case": "Abl",
        "Number": "Sing",
        "Person": "2",
        "Polite": "Infm",
        "PronType": "Prs",
    },
    "PRON_Case=Dat|Definite=Def|Distance=Dist|Number=Sing|PronType=Dem": {
        POS: PRON,
        "Case": "Dat",
        "Definite": "Def",
        "Distance": "Dist",
        "Number": "Sing",
        "PronType": "Dem",
    },
    "PRON_Case=Dat|Definite=Def|Number=Sing|Person=3|PronType=Prs": {
        POS: PRON,
        "Case": "Dat",
        "Definite": "Def",
        "Number": "Sing",
        "Person": "3",
        "PronType": "Prs",
    },
    "PRON_Case=Dat|Definite=Ind|Number=Sing|PronType=Int": {
        POS: PRON,
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Sing",
        "PronType": "Int",
    },
    "PRON_Case=Dat|Distance=Dist|Number=Sing|PronType=Dem": {
        POS: PRON,
        "Case": "Dat",
        "Distance": "Dist",
        "Number": "Sing",
        "PronType": "Dem",
    },
    "PRON_Case=Dat|Distance=Med|Number=Plur|PronType=Dem": {
        POS: PRON,
        "Case": "Dat",
        "Distance": "Med",
        "Number": "Plur",
        "PronType": "Dem",
    },
    "PRON_Case=Dat|Number=Plur|Person=1|PronType=Prs": {
        POS: PRON,
        "Case": "Dat",
        "Number": "Plur",
        "Person": "1",
        "PronType": "Prs",
    },
    "PRON_Case=Dat|Number=Plur|Person=2|Polite=Infm|PronType=Prs": {
        POS: PRON,
        "Case": "Dat",
        "Number": "Plur",
        "Person": "2",
        "Polite": "Infm",
        "PronType": "Prs",
    },
    "PRON_Case=Dat|Number=Plur|Person=3|PronType=Emp|Reflex=Yes": {
        POS: PRON,
        "Case": "Dat",
        "Number": "Plur",
        "Person": "3",
        "PronType": "Emp",
        "Reflex": "Yes",
    },
    "PRON_Case=Dat|Number=Plur|Person=3|PronType=Prs": {
        POS: PRON,
        "Case": "Dat",
        "Number": "Plur",
        "Person": "3",
        "PronType": "Prs",
    },
    "PRON_Case=Dat|Number=Plur|PronType=Rcp": {
        POS: PRON,
        "Case": "Dat",
        "Number": "Plur",
        "PronType": "Rcp",
    },
    "PRON_Case=Dat|Number=Sing|Person=1|PronType=Prs": {
        POS: PRON,
        "Case": "Dat",
        "Number": "Sing",
        "Person": "1",
        "PronType": "Prs",
    },
    "PRON_Case=Dat|Number=Sing|Person=2|Polite=Infm|PronType=Prs": {
        POS: PRON,
        "Case": "Dat",
        "Number": "Sing",
        "Person": "2",
        "Polite": "Infm",
        "PronType": "Prs",
    },
    "PRON_Case=Dat|Number=Sing|Person=3|PronType=Emp": {
        POS: PRON,
        "Case": "Dat",
        "Number": "Sing",
        "Person": "3",
        "PronType": "Emp",
    },
    "PRON_Case=Dat|Number=Sing|Person=3|PronType=Emp|Reflex=Yes": {
        POS: PRON,
        "Case": "Dat",
        "Number": "Sing",
        "Person": "3",
        "PronType": "Emp",
        "Reflex": "Yes",
    },
    "PRON_Case=Dat|Number=Sing|PronType=Int": {
        POS: PRON,
        "Case": "Dat",
        "Number": "Sing",
        "PronType": "Int",
    },
    "PRON_Case=Dat|Number=Sing|PronType=Rel": {
        POS: PRON,
        "Case": "Dat",
        "Number": "Sing",
        "PronType": "Rel",
    },
    "PRON_Case=Dat|PronType=Tot": {POS: PRON, "Case": "Dat", "PronType": "Tot"},
    "PRON_Case=Gen|Distance=Med|Number=Sing|PronType=Dem": {
        POS: PRON,
        "Case": "Gen",
        "Distance": "Med",
        "Number": "Sing",
        "PronType": "Dem",
    },
    "PRON_Case=Gen|Number=Plur|Person=1|PronType=Prs": {
        POS: PRON,
        "Case": "Gen",
        "Number": "Plur",
        "Person": "1",
        "PronType": "Prs",
    },
    "PRON_Case=Gen|Number=Sing|Person=2|PronType=Prs": {
        POS: PRON,
        "Case": "Gen",
        "Number": "Sing",
        "Person": "2",
        "PronType": "Prs",
    },
    "PRON_Case=Gen|Number=Sing|Person=3|PronType=Prs": {
        POS: PRON,
        "Case": "Gen",
        "Number": "Sing",
        "Person": "3",
        "PronType": "Prs",
    },
    "PRON_Case=Gen|PronType=Tot": {POS: PRON, "Case": "Gen", "PronType": "Tot"},
    "PRON_Case=Ins|Definite=Ind|Number=Sing|PronType=Rel": {
        POS: PRON,
        "Case": "Ins",
        "Definite": "Ind",
        "Number": "Sing",
        "PronType": "Rel",
    },
    "PRON_Case=Ins|Distance=Med|Number=Sing|PronType=Dem": {
        POS: PRON,
        "Case": "Ins",
        "Distance": "Med",
        "Number": "Sing",
        "PronType": "Dem",
    },
    "PRON_Case=Loc|Definite=Ind|Number=Sing|PronType=Rel": {
        POS: PRON,
        "Case": "Loc",
        "Definite": "Ind",
        "Number": "Sing",
        "PronType": "Rel",
    },
    "PRON_Case=Loc|Distance=Med|Number=Sing|PronType=Dem": {
        POS: PRON,
        "Case": "Loc",
        "Distance": "Med",
        "Number": "Sing",
        "PronType": "Dem",
    },
    "PRON_Case=Nom|Definite=Def|Distance=Dist|Number=Plur|PronType=Dem": {
        POS: PRON,
        "Case": "Nom",
        "Definite": "Def",
        "Distance": "Dist",
        "Number": "Plur",
        "PronType": "Dem",
    },
    "PRON_Case=Nom|Definite=Def|Distance=Med|Number=Sing|PronType=Dem|Style=Coll": {
        POS: PRON,
        "Case": "Nom",
        "Definite": "Def",
        "Distance": "Med",
        "Number": "Sing",
        "PronType": "Dem",
        "Style": "Coll",
    },
    "PRON_Case=Nom|Definite=Def|Number=Sing|PronType=Int": {
        POS: PRON,
        "Case": "Nom",
        "Definite": "Def",
        "Number": "Sing",
        "PronType": "Int",
    },
    "PRON_Case=Nom|Definite=Def|Number=Sing|PronType=Rel": {
        POS: PRON,
        "Case": "Nom",
        "Definite": "Def",
        "Number": "Sing",
        "PronType": "Rel",
    },
    "PRON_Case=Nom|Definite=Ind|Number=Sing|PronType=Int": {
        POS: PRON,
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Sing",
        "PronType": "Int",
    },
    "PRON_Case=Nom|Definite=Ind|Number=Sing|PronType=Neg": {
        POS: PRON,
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Sing",
        "PronType": "Neg",
    },
    "PRON_Case=Nom|Definite=Ind|Number=Sing|PronType=Rel": {
        POS: PRON,
        "Case": "Nom",
        "Definite": "Ind",
        "Number": "Sing",
        "PronType": "Rel",
    },
    "PRON_Case=Nom|Distance=Dist|Number=Plur|Person=1|PronType=Dem": {
        POS: PRON,
        "Case": "Nom",
        "Distance": "Dist",
        "Number": "Plur",
        "Person": "1",
        "PronType": "Dem",
    },
    "PRON_Case=Nom|Distance=Med|Number=Plur|PronType=Dem": {
        POS: PRON,
        "Case": "Nom",
        "Distance": "Med",
        "Number": "Plur",
        "PronType": "Dem",
    },
    "PRON_Case=Nom|Distance=Med|Number=Sing|PronType=Dem": {
        POS: PRON,
        "Case": "Nom",
        "Distance": "Med",
        "Number": "Sing",
        "PronType": "Dem",
    },
    "PRON_Case=Nom|Distance=Prox|Number=Sing|PronType=Dem": {
        POS: PRON,
        "Case": "Nom",
        "Distance": "Prox",
        "Number": "Sing",
        "PronType": "Dem",
    },
    "PRON_Case=Nom|Number=Plur|Person=1|PronType=Prs": {
        POS: PRON,
        "Case": "Nom",
        "Number": "Plur",
        "Person": "1",
        "PronType": "Prs",
    },
    "PRON_Case=Nom|Number=Plur|Person=3|PronType=Emp": {
        POS: PRON,
        "Case": "Nom",
        "Number": "Plur",
        "Person": "3",
        "PronType": "Emp",
    },
    "PRON_Case=Nom|Number=Plur|Person=3|PronType=Prs": {
        POS: PRON,
        "Case": "Nom",
        "Number": "Plur",
        "Person": "3",
        "PronType": "Prs",
    },
    "PRON_Case=Nom|Number=Plur|PronType=Rel": {
        POS: PRON,
        "Case": "Nom",
        "Number": "Plur",
        "PronType": "Rel",
    },
    "PRON_Case=Nom|Number=Sing|Number=Plur|Person=3|Person=1|PronType=Emp": {
        POS: PRON,
        "Case": "Nom",
        # "Number": "Sing",
        "Number": "Plur",
        # "Person": "3",
        "Person": "1",
        "PronType": "Emp",
    },
    "PRON_Case=Nom|Number=Sing|Person=1|PronType=Int": {
        POS: PRON,
        "Case": "Nom",
        "Number": "Sing",
        "Person": "1",
        "PronType": "Int",
    },
    "PRON_Case=Nom|Number=Sing|Person=1|PronType=Prs": {
        POS: PRON,
        "Case": "Nom",
        "Number": "Sing",
        "Person": "1",
        "PronType": "Prs",
    },
    "PRON_Case=Nom|Number=Sing|Person=2|Polite=Infm|PronType=Prs": {
        POS: PRON,
        "Case": "Nom",
        "Number": "Sing",
        "Person": "2",
        "Polite": "Infm",
        "PronType": "Prs",
    },
    "PRON_Case=Nom|Number=Sing|Person=3|PronType=Emp": {
        POS: PRON,
        "Case": "Nom",
        "Number": "Sing",
        "Person": "3",
        "PronType": "Emp",
    },
    "PRON_Case=Nom|Number=Sing|Person=3|PronType=Prs": {
        POS: PRON,
        "Case": "Nom",
        "Number": "Sing",
        "Person": "3",
        "PronType": "Prs",
    },
    "PRON_Case=Nom|Number=Sing|PronType=Int": {
        POS: PRON,
        "Case": "Nom",
        "Number": "Sing",
        "PronType": "Int",
    },
    "PRON_Case=Nom|Number=Sing|PronType=Rel": {
        POS: PRON,
        "Case": "Nom",
        "Number": "Sing",
        "PronType": "Rel",
    },
    "PRON_Case=Nom|Person=1|PronType=Tot": {
        POS: PRON,
        "Case": "Nom",
        "Person": "1",
        "PronType": "Tot",
    },
    "PRON_Case=Nom|PronType=Ind": {POS: PRON, "Case": "Nom", "PronType": "Ind"},
    "PRON_Case=Nom|PronType=Tot": {POS: PRON, "Case": "Nom", "PronType": "Tot"},
    "PRON_Distance=Dist|Number=Sing|PronType=Dem": {
        POS: PRON,
        "Distance": "Dist",
        "Number": "Sing",
        "PronType": "Dem",
    },
    "PRON_Distance=Med|PronType=Dem|Style=Coll": {
        POS: PRON,
        "Distance": "Med",
        "PronType": "Dem",
        "Style": "Coll",
    },
    "PRON_Distance=Prox|PronType=Dem|Style=Coll": {
        POS: PRON,
        "Distance": "Prox",
        "PronType": "Dem",
        "Style": "Coll",
    },
    "PRON_Number=Plur|PronType=Rel": {POS: PRON, "Number": "Plur", "PronType": "Rel"},
    "PROPN_Abbr=Yes|Animacy=Hum|Case=Nom|Definite=Ind|NameType=Giv|Number=Sing": {
        POS: PROPN,
        "Abbr": "Yes",
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Ind",
        "NameType": "Giv",
        "Number": "Sing",
    },
    "PROPN_Abbr=Yes|Animacy=Nhum|Case=Nom|Definite=Ind|NameType=Com|Number=Sing": {
        POS: PROPN,
        "Abbr": "Yes",
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Ind",
        "NameType": "Com",
        "Number": "Sing",
    },
    "PROPN_Animacy=Hum|Case=Dat|Definite=Def|NameType=Sur|Number=Sing": {
        POS: PROPN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Definite": "Def",
        "NameType": "Sur",
        "Number": "Sing",
    },
    "PROPN_Animacy=Hum|Case=Dat|Definite=Ind|NameType=Prs|Number=Sing": {
        POS: PROPN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Definite": "Ind",
        "NameType": "Prs",
        "Number": "Sing",
    },
    "PROPN_Animacy=Hum|Case=Dat|Definite=Ind|NameType=Sur|Number=Sing": {
        POS: PROPN,
        "Animacy": "Hum",
        "Case": "Dat",
        "Definite": "Ind",
        "NameType": "Sur",
        "Number": "Sing",
    },
    "PROPN_Animacy=Hum|Case=Nom|Definite=Def|NameType=Giv|Number=Sing": {
        POS: PROPN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Def",
        "NameType": "Giv",
        "Number": "Sing",
    },
    "PROPN_Animacy=Hum|Case=Nom|Definite=Def|NameType=Sur|Number=Sing": {
        POS: PROPN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Def",
        "NameType": "Sur",
        "Number": "Sing",
    },
    "PROPN_Animacy=Hum|Case=Nom|Definite=Ind|NameType=Giv|Number=Sing": {
        POS: PROPN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Ind",
        "NameType": "Giv",
        "Number": "Sing",
    },
    "PROPN_Animacy=Hum|Case=Nom|Definite=Ind|NameType=Sur|Number=Sing": {
        POS: PROPN,
        "Animacy": "Hum",
        "Case": "Nom",
        "Definite": "Ind",
        "NameType": "Sur",
        "Number": "Sing",
    },
    "PROPN_Animacy=Nhum|Case=Abl|Definite=Ind|NameType=Geo|Number=Coll": {
        POS: PROPN,
        "Animacy": "Nhum",
        "Case": "Abl",
        "Definite": "Ind",
        "NameType": "Geo",
        "Number": "Coll",
    },
    "PROPN_Animacy=Nhum|Case=Abl|Definite=Ind|NameType=Geo|Number=Sing": {
        POS: PROPN,
        "Animacy": "Nhum",
        "Case": "Abl",
        "Definite": "Ind",
        "NameType": "Geo",
        "Number": "Sing",
    },
    "PROPN_Animacy=Nhum|Case=Abl|Definite=Ind|Number=Plur": {
        POS: PROPN,
        "Animacy": "Nhum",
        "Case": "Abl",
        "Definite": "Ind",
        "Number": "Plur",
    },
    "PROPN_Animacy=Nhum|Case=Dat|Definite=Ind|NameType=Geo|Number=Sing": {
        POS: PROPN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Ind",
        "NameType": "Geo",
        "Number": "Sing",
    },
    "PROPN_Animacy=Nhum|Case=Dat|Definite=Ind|NameType=Geo|Number=Sing|Style=Coll": {
        POS: PROPN,
        "Animacy": "Nhum",
        "Case": "Dat",
        "Definite": "Ind",
        "NameType": "Geo",
        "Number": "Sing",
        "Style": "Coll",
    },
    "PROPN_Animacy=Nhum|Case=Loc|Definite=Ind|NameType=Geo|Number=Sing": {
        POS: PROPN,
        "Animacy": "Nhum",
        "Case": "Loc",
        "Definite": "Ind",
        "NameType": "Geo",
        "Number": "Sing",
    },
    "PROPN_Animacy=Nhum|Case=Nom|Definite=Def|NameType=Geo|Number=Sing": {
        POS: PROPN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Def",
        "NameType": "Geo",
        "Number": "Sing",
    },
    "PROPN_Animacy=Nhum|Case=Nom|Definite=Def|NameType=Pro|Number=Sing|Style=Coll": {
        POS: PROPN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Def",
        "NameType": "Pro",
        "Number": "Sing",
        "Style": "Coll",
    },
    "PROPN_Animacy=Nhum|Case=Nom|Definite=Ind|NameType=Geo|Number=Coll": {
        POS: PROPN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Ind",
        "NameType": "Geo",
        "Number": "Coll",
    },
    "PROPN_Animacy=Nhum|Case=Nom|Definite=Ind|NameType=Geo|Number=Sing": {
        POS: PROPN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Ind",
        "NameType": "Geo",
        "Number": "Sing",
    },
    "PROPN_Animacy=Nhum|Case=Nom|Definite=Ind|NameType=Geo|Number=Sing|Style=Vrnc": {
        POS: PROPN,
        "Animacy": "Nhum",
        "Case": "Nom",
        "Definite": "Ind",
        "NameType": "Geo",
        "Number": "Sing",
        "Style": "Vrnc",
    },
    "SCONJ_Style=Coll": {POS: SCONJ, "Style": "Coll"},
    "VERB_Aspect=Dur|Polarity=Neg|Subcat=Intr|VerbForm=Part|Voice=Pass": {
        POS: VERB,
        "Aspect": "Dur",
        "Polarity": "Neg",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Pass",
    },
    "VERB_Aspect=Dur|Polarity=Pos|Subcat=Intr|VerbForm=Part|Voice=Mid": {
        POS: VERB,
        "Aspect": "Dur",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Mid",
    },
    "VERB_Aspect=Dur|Polarity=Pos|Subcat=Intr|VerbForm=Part|Voice=Pass": {
        POS: VERB,
        "Aspect": "Dur",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Pass",
    },
    "VERB_Aspect=Dur|Polarity=Pos|Subcat=Tran|VerbForm=Part|Voice=Act": {
        POS: VERB,
        "Aspect": "Dur",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "VerbForm": "Part",
        "Voice": "Act",
    },
    "VERB_Aspect=Dur|Polarity=Pos|Subcat=Tran|VerbForm=Part|Voice=Mid": {
        POS: VERB,
        "Aspect": "Dur",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "VerbForm": "Part",
        "Voice": "Mid",
    },
    "VERB_Aspect=Imp|Mood=Ind|Number=Plur|Person=1|Polarity=Neg|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "1",
        "Polarity": "Neg",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Imp|Mood=Ind|Number=Plur|Person=1|Polarity=Pos|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "1",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Imp|Mood=Ind|Number=Plur|Person=3|Polarity=Neg|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Neg",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Imp|Mood=Ind|Number=Plur|Person=3|Polarity=Pos|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Polarity=Pos|Subcat=Tran|Tense=Imp|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Imp",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Imp|Mood=Ind|Number=Sing|Person=2|Polarity=Neg|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "2",
        "Polarity": "Neg",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Polarity=Neg|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Neg",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Polarity=Neg|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Neg",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Polarity=Pos|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Polarity=Pos|Subcat=Tran|Tense=Imp|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Imp",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Polarity=Pos|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Imp",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Imp|Style=Coll|Subcat=Intr|VerbForm=Part|Voice=Mid": {
        POS: VERB,
        "Aspect": "Imp",
        "Style": "Coll",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Mid",
    },
    "VERB_Aspect=Imp|Style=Vrnc|Subcat=Intr|VerbForm=Part|Voice=Mid": {
        POS: VERB,
        "Aspect": "Imp",
        "Style": "Vrnc",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Mid",
    },
    "VERB_Aspect=Imp|Subcat=Intr|VerbForm=Part": {
        POS: VERB,
        "Aspect": "Imp",
        "Subcat": "Intr",
        "VerbForm": "Part",
    },
    "VERB_Aspect=Imp|Subcat=Intr|VerbForm=Part|Voice=Act": {
        POS: VERB,
        "Aspect": "Imp",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Act",
    },
    "VERB_Aspect=Imp|Subcat=Intr|VerbForm=Part|Voice=Mid": {
        POS: VERB,
        "Aspect": "Imp",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Mid",
    },
    "VERB_Aspect=Imp|Subcat=Intr|VerbForm=Part|Voice=Pass": {
        POS: VERB,
        "Aspect": "Imp",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Pass",
    },
    "VERB_Aspect=Imp|Subcat=Tran|VerbForm=Part|Voice=Act": {
        POS: VERB,
        "Aspect": "Imp",
        "Subcat": "Tran",
        "VerbForm": "Part",
        "Voice": "Act",
    },
    "VERB_Aspect=Imp|Subcat=Tran|VerbForm=Part|Voice=Cau": {
        POS: VERB,
        "Aspect": "Imp",
        "Subcat": "Tran",
        "VerbForm": "Part",
        "Voice": "Cau",
    },
    "VERB_Aspect=Iter|Case=Ins|Definite=Ind|Number=Coll|Polarity=Pos|Subcat=Intr|VerbForm=Gdv|Voice=Mid": {
        POS: VERB,
        "Aspect": "Iter",
        "Case": "Ins",
        "Definite": "Ind",
        "Number": "Coll",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Gdv",
        "Voice": "Mid",
    },
    "VERB_Aspect=Iter|Case=Ins|Definite=Ind|Number=Coll|Polarity=Pos|Subcat=Tran|VerbForm=Gdv|Voice=Act": {
        POS: VERB,
        "Aspect": "Iter",
        "Case": "Ins",
        "Definite": "Ind",
        "Number": "Coll",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "VerbForm": "Gdv",
        "Voice": "Act",
    },
    "VERB_Aspect=Iter": {POS: VERB, "Aspect": "Iter"},
    "VERB_Aspect=Perf|Mood=Ind|Number=Plur|Person=3|Polarity=Neg|Subcat=Intr|Tense=Past|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Neg",
        "Subcat": "Intr",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Perf|Mood=Ind|Number=Plur|Person=3|Polarity=Pos|Subcat=Intr|Tense=Past|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Perf|Mood=Ind|Number=Plur|Person=3|Polarity=Pos|Subcat=Tran|Tense=Past|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Perf|Mood=Ind|Number=Sing|Person=1|Polarity=Neg|Subcat=Intr|Tense=Past|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Neg",
        "Subcat": "Intr",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Perf|Mood=Ind|Number=Sing|Person=1|Polarity=Pos|Style=Vrnc|Subcat=Tran|Tense=Past|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Pos",
        "Style": "Vrnc",
        "Subcat": "Tran",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Perf|Mood=Ind|Number=Sing|Person=1|Polarity=Pos|Subcat=Intr|Tense=Past|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Perf|Mood=Ind|Number=Sing|Person=1|Polarity=Pos|Subcat=Tran|Tense=Past|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Perf|Mood=Ind|Number=Sing|Person=2|Polarity=Pos|Subcat=Tran|Tense=Past|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "2",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Perf|Mood=Ind|Number=Sing|Person=3|Polarity=Neg|Style=Vrnc|Subcat=Intr|Tense=Past|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Neg",
        "Style": "Vrnc",
        "Subcat": "Intr",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Perf|Mood=Ind|Number=Sing|Person=3|Polarity=Neg|Subcat=Tran|Tense=Past|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Neg",
        "Subcat": "Tran",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Perf|Mood=Ind|Number=Sing|Person=3|Polarity=Pos|Subcat=Intr|Tense=Past|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Perf|Mood=Ind|Number=Sing|Person=3|Polarity=Pos|Subcat=Tran|Tense=Past|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Perf|Mood=Ind|Number=Sing|Person=3|Polarity=Pos|Subcat=Tran|Tense=Past|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Perf",
        "Mood": "Ind",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Past",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Perf|Polarity=Neg|Subcat=Intr|VerbForm=Part|Voice=Pass": {
        POS: VERB,
        "Aspect": "Perf",
        "Polarity": "Neg",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Pass",
    },
    "VERB_Aspect=Perf|Polarity=Pos|Subcat=Intr|VerbForm=Part|Voice=Mid": {
        POS: VERB,
        "Aspect": "Perf",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Mid",
    },
    "VERB_Aspect=Perf|Polarity=Pos|Subcat=Intr|VerbForm=Part|Voice=Pass": {
        POS: VERB,
        "Aspect": "Perf",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Pass",
    },
    "VERB_Aspect=Perf|Polarity=Pos|Subcat=Tran|VerbForm=Part|Voice=Act": {
        POS: VERB,
        "Aspect": "Perf",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "VerbForm": "Part",
        "Voice": "Act",
    },
    "VERB_Aspect=Perf|Polarity=Pos|Subcat=Tran|VerbForm=Part|Voice=Pass": {
        POS: VERB,
        "Aspect": "Perf",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "VerbForm": "Part",
        "Voice": "Pass",
    },
    "VERB_Aspect=Perf|Polarity=Pos|VerbForm=Part|Voice=Act": {
        POS: VERB,
        "Aspect": "Perf",
        "Polarity": "Pos",
        "VerbForm": "Part",
        "Voice": "Act",
    },
    "VERB_Aspect=Perf|Subcat=Intr|VerbForm=Part|Voice=Mid": {
        POS: VERB,
        "Aspect": "Perf",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Mid",
    },
    "VERB_Aspect=Perf|Subcat=Intr|VerbForm=Part|Voice=Pass": {
        POS: VERB,
        "Aspect": "Perf",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Pass",
    },
    "VERB_Aspect=Perf|Subcat=Tran|VerbForm=Part|Voice=Act": {
        POS: VERB,
        "Aspect": "Perf",
        "Subcat": "Tran",
        "VerbForm": "Part",
        "Voice": "Act",
    },
    "VERB_Aspect=Perf|Subcat=Tran|VerbForm=Part|Voice=Cau": {
        POS: VERB,
        "Aspect": "Perf",
        "Subcat": "Tran",
        "VerbForm": "Part",
        "Voice": "Cau",
    },
    "VERB_Aspect=Prog|Subcat=Intr|VerbForm=Conv|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prog",
        "Subcat": "Intr",
        "VerbForm": "Conv",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Connegative=Yes|Mood=Cnd|Subcat=Tran|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Connegative": "Yes",
        "Mood": "Cnd",
        "Subcat": "Tran",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Cnd|Number=Plur|Person=3|Polarity=Pos|Style=Vrnc|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Cnd",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Pos",
        "Style": "Vrnc",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Cnd|Number=Plur|Person=3|Polarity=Pos|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Cnd",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Mood=Cnd|Number=Sing|Person=1|Polarity=Pos|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Cnd",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Mood=Cnd|Number=Sing|Person=2|Polarity=Pos|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Cnd",
        "Number": "Sing",
        "Person": "2",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Cnd|Number=Sing|Person=3|Polarity=Pos|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Cnd",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Mood=Cnd|Number=Sing|Person=3|Polarity=Pos|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Pass": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Cnd",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Pass",
    },
    "VERB_Aspect=Prosp|Mood=Cnd|Number=Sing|Person=3|Polarity=Pos|Subcat=Tran|Tense=Imp|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Cnd",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Imp",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Cnd|Number=Sing|Person=3|Polarity=Pos|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Cnd",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Imp|Number=Sing|Person=2|Subcat=Intr|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Imp",
        "Number": "Sing",
        "Person": "2",
        "Subcat": "Intr",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Mood=Imp|Number=Sing|Person=2|Subcat=Tran|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Imp",
        "Number": "Sing",
        "Person": "2",
        "Subcat": "Tran",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Plur|Person=1|Polarity=Pos|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Plur",
        "Person": "1",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Plur|Person=3|Polarity=Neg|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Neg",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Plur|Person=3|Polarity=Pos|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Plur",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Sing|Person=1|Polarity=Neg|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Neg",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Sing|Person=1|Polarity=Neg|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Neg",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Sing|Person=1|Polarity=Pos|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Sing|Person=1|Polarity=Pos|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Sing",
        "Person": "1",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Sing|Person=2|Polarity=Pos|Subcat=Tran|Tense=Imp|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Sing",
        "Person": "2",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Imp",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Sing|Person=2|Polarity=Pos|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Sing",
        "Person": "2",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Sing|Person=3|Polarity=Pos|Subcat=Intr|Tense=Imp|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Imp",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Sing|Person=3|Polarity=Pos|Subcat=Intr|Tense=Pres|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Sing|Person=3|Polarity=Pos|Subcat=Intr|VerbForm=Fin|Voice=Pass": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Fin",
        "Voice": "Pass",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Sing|Person=3|Polarity=Pos|Subcat=Tran|Tense=Imp|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Imp",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Number=Sing|Person=3|Polarity=Pos|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Number": "Sing",
        "Person": "3",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Mood=Sub|Person=1|Polarity=Neg|Subcat=Tran|Tense=Pres|VerbForm=Fin|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Mood": "Sub",
        "Person": "1",
        "Polarity": "Neg",
        "Subcat": "Tran",
        "Tense": "Pres",
        "VerbForm": "Fin",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Polarity=Pos|Subcat=Intr|VerbForm=Part|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prosp",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Polarity=Pos|Subcat=Tran|VerbForm=Part|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "VerbForm": "Part",
        "Voice": "Act",
    },
    "VERB_Aspect=Prosp|Subcat=Intr|VerbForm=Part|Voice=Mid": {
        POS: VERB,
        "Aspect": "Prosp",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Mid",
    },
    "VERB_Aspect=Prosp|Subcat=Intr|VerbForm=Part|Voice=Pass": {
        POS: VERB,
        "Aspect": "Prosp",
        "Subcat": "Intr",
        "VerbForm": "Part",
        "Voice": "Pass",
    },
    "VERB_Aspect=Prosp|Subcat=Tran|VerbForm=Part|Voice=Act": {
        POS: VERB,
        "Aspect": "Prosp",
        "Subcat": "Tran",
        "VerbForm": "Part",
        "Voice": "Act",
    },
    "VERB_Case=Abl|Definite=Ind|Number=Coll|Polarity=Pos|Subcat=Intr|VerbForm=Gdv|Voice=Mid": {
        POS: VERB,
        "Case": "Abl",
        "Definite": "Ind",
        "Number": "Coll",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Gdv",
        "Voice": "Mid",
    },
    "VERB_Case=Abl|Definite=Ind|Number=Coll|Polarity=Pos|Subcat=Intr|VerbForm=Gdv|Voice=Pass": {
        POS: VERB,
        "Case": "Abl",
        "Definite": "Ind",
        "Number": "Coll",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Gdv",
        "Voice": "Pass",
    },
    "VERB_Case=Abl|Definite=Ind|Number=Coll|Polarity=Pos|Subcat=Tran|VerbForm=Gdv|Voice=Act": {
        POS: VERB,
        "Case": "Abl",
        "Definite": "Ind",
        "Number": "Coll",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "VerbForm": "Gdv",
        "Voice": "Act",
    },
    "VERB_Case=Dat|Definite=Def|Number=Coll|Polarity=Pos|Subcat=Intr|VerbForm=Gdv|Voice=Mid": {
        POS: VERB,
        "Case": "Dat",
        "Definite": "Def",
        "Number": "Coll",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Gdv",
        "Voice": "Mid",
    },
    "VERB_Case=Dat|Definite=Ind|Number=Coll|Polarity=Neg|Subcat=Intr|VerbForm=Gdv|Voice=Pass": {
        POS: VERB,
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Coll",
        "Polarity": "Neg",
        "Subcat": "Intr",
        "VerbForm": "Gdv",
        "Voice": "Pass",
    },
    "VERB_Case=Dat|Definite=Ind|Number=Coll|Polarity=Pos|Subcat=Intr|VerbForm=Gdv|Voice=Mid": {
        POS: VERB,
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Coll",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Gdv",
        "Voice": "Mid",
    },
    "VERB_Case=Dat|Definite=Ind|Number=Coll|Polarity=Pos|Subcat=Tran|VerbForm=Gdv|Voice=Act": {
        POS: VERB,
        "Case": "Dat",
        "Definite": "Ind",
        "Number": "Coll",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "VerbForm": "Gdv",
        "Voice": "Act",
    },
    "VERB_Case=Ins|Definite=Ind|Number=Coll|Polarity=Pos|Subcat=Intr|VerbForm=Gdv|Voice=Mid": {
        POS: VERB,
        "Case": "Ins",
        "Definite": "Ind",
        "Number": "Coll",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Gdv",
        "Voice": "Mid",
    },
    "VERB_Case=Ins|Definite=Ind|Number=Coll|Polarity=Pos|Subcat=Tran|VerbForm=Gdv|Voice=Act": {
        POS: VERB,
        "Case": "Ins",
        "Definite": "Ind",
        "Number": "Coll",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "VerbForm": "Gdv",
        "Voice": "Act",
    },
    "VERB_Case=Nom|Definite=Def|Number=Coll|Polarity=Pos|Subcat=Intr|VerbForm=Gdv|Voice=Mid": {
        POS: VERB,
        "Case": "Nom",
        "Definite": "Def",
        "Number": "Coll",
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Gdv",
        "Voice": "Mid",
    },
    "VERB_Case=Nom|Definite=Def|Number=Coll|Polarity=Pos|Subcat=Tran|VerbForm=Gdv|Voice=Act": {
        POS: VERB,
        "Case": "Nom",
        "Definite": "Def",
        "Number": "Coll",
        "Polarity": "Pos",
        "Subcat": "Tran",
        "VerbForm": "Gdv",
        "Voice": "Act",
    },
    "VERB_Mood=Imp|Number=Sing|Person=2|Subcat=Intr|VerbForm=Fin|Voice=Mid": {
        POS: VERB,
        "Mood": "Imp",
        "Number": "Sing",
        "Person": "2",
        "Subcat": "Intr",
        "VerbForm": "Fin",
        "Voice": "Mid",
    },
    "VERB_Polarity=Neg|Subcat=Intr|VerbForm=Inf|Voice=Mid": {
        POS: VERB,
        "Polarity": "Neg",
        "Subcat": "Intr",
        "VerbForm": "Inf",
        "Voice": "Mid",
    },
    "VERB_Polarity=Pos|Style=Coll|Subcat=Tran|VerbForm=Inf|Voice=Act": {
        POS: VERB,
        "Polarity": "Pos",
        "Style": "Coll",
        "Subcat": "Tran",
        "VerbForm": "Inf",
        "Voice": "Act",
    },
    "VERB_Polarity=Pos|Style=Vrnc|Subcat=Tran|VerbForm=Inf|Voice=Act": {
        POS: VERB,
        "Polarity": "Pos",
        "Style": "Vrnc",
        "Subcat": "Tran",
        "VerbForm": "Inf",
        "Voice": "Act",
    },
    "VERB_Polarity=Pos|Subcat=Intr|VerbForm=Inf|Voice=Mid": {
        POS: VERB,
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Inf",
        "Voice": "Mid",
    },
    "VERB_Polarity=Pos|Subcat=Intr|VerbForm=Inf|Voice=Pass": {
        POS: VERB,
        "Polarity": "Pos",
        "Subcat": "Intr",
        "VerbForm": "Inf",
        "Voice": "Pass",
    },
    "VERB_Polarity=Pos|Subcat=Tran|Typo=Yes|VerbForm=Inf|Voice=Act": {
        POS: VERB,
        "Polarity": "Pos",
        "Subcat": "Tran",
        "Typo": "Yes",
        "VerbForm": "Inf",
        "Voice": "Act",
    },
    "VERB_Polarity=Pos|Subcat=Tran|VerbForm=Inf|Voice=Act": {
        POS: VERB,
        "Polarity": "Pos",
        "Subcat": "Tran",
        "VerbForm": "Inf",
        "Voice": "Act",
    },
    "X_Foreign=Yes": {POS: X, "Foreign": "Yes"},
    "X_Style=Vrnc": {POS: X, "Style": "Vrnc"},
}



# coding: utf8
from __future__ import unicode_literals

# Source: https://github.com/Xangis/extra-stopwords (MIT License)

STOP_WORDS = set(
    """
అందరూ
అందుబాటులో
అడగండి
అడగడం
అడ్డంగా
అనుగుణంగా
అనుమతించు
అనుమతిస్తుంది
అయితే
ఇప్పటికే
ఉన్నారు
ఎక్కడైనా
ఎప్పుడు
ఎవరైనా
ఎవరో ఒకరు
ఏ
ఏదైనా
ఏమైనప్పటికి
ఏమైనప్పటికి
ఒక
ఒక ప్రక్కన
కనిపిస్తాయి
కాదు
కాదు
కూడా
గా
గురించి
చుట్టూ
చేయగలిగింది
తగిన
తర్వాత
తర్వాత
దాదాపు
దూరంగా
నిజంగా
పై
ప్రకారం
మధ్య
మధ్య
మరియు
మరొక
మళ్ళీ
మాత్రమే
మెచ్చుకో
వద్ద
వద్ద
వెంట
వేరుగా
వ్యతిరేకంగా
సంబంధం
""".split()
)



# coding: utf8
from __future__ import unicode_literals

import pytest
from spacy.tokens import Span
from spacy.language import Language
from spacy.pipeline import EntityRuler
from spacy.errors import MatchPatternError


@pytest.fixture
def nlp():
    return Language()


@pytest.fixture
def patterns():
    return [
        {"label": "HELLO", "pattern": "hello world"},
        {"label": "BYE", "pattern": [{"LOWER": "bye"}, {"LOWER": "bye"}]},
        {"label": "HELLO", "pattern": [{"ORTH": "HELLO"}]},
        {"label": "COMPLEX", "pattern": [{"ORTH": "foo", "OP": "*"}]},
        {"label": "TECH_ORG", "pattern": "Apple", "id": "a1"},
        {"label": "TECH_ORG", "pattern": "Microsoft", "id": "a2"},
    ]


@pytest.fixture
def add_ent():
    def add_ent_component(doc):
        doc.ents = [Span(doc, 0, 3, label=doc.vocab.strings["ORG"])]
        return doc

    return add_ent_component


def test_entity_ruler_init(nlp, patterns):
    ruler = EntityRuler(nlp, patterns=patterns)
    assert len(ruler) == len(patterns)
    assert len(ruler.labels) == 4
    assert "HELLO" in ruler
    assert "BYE" in ruler
    nlp.add_pipe(ruler)
    doc = nlp("hello world bye bye")
    assert len(doc.ents) == 2
    assert doc.ents[0].label_ == "HELLO"
    assert doc.ents[1].label_ == "BYE"


def test_entity_ruler_existing(nlp, patterns, add_ent):
    ruler = EntityRuler(nlp, patterns=patterns)
    nlp.add_pipe(add_ent)
    nlp.add_pipe(ruler)
    doc = nlp("OH HELLO WORLD bye bye")
    assert len(doc.ents) == 2
    assert doc.ents[0].label_ == "ORG"
    assert doc.ents[1].label_ == "BYE"


def test_entity_ruler_existing_overwrite(nlp, patterns, add_ent):
    ruler = EntityRuler(nlp, patterns=patterns, overwrite_ents=True)
    nlp.add_pipe(add_ent)
    nlp.add_pipe(ruler)
    doc = nlp("OH HELLO WORLD bye bye")
    assert len(doc.ents) == 2
    assert doc.ents[0].label_ == "HELLO"
    assert doc.ents[0].text == "HELLO"
    assert doc.ents[1].label_ == "BYE"


def test_entity_ruler_existing_complex(nlp, patterns, add_ent):
    ruler = EntityRuler(nlp, patterns=patterns, overwrite_ents=True)
    nlp.add_pipe(add_ent)
    nlp.add_pipe(ruler)
    doc = nlp("foo foo bye bye")
    assert len(doc.ents) == 2
    assert doc.ents[0].label_ == "COMPLEX"
    assert doc.ents[1].label_ == "BYE"
    assert len(doc.ents[0]) == 2
    assert len(doc.ents[1]) == 2


def test_entity_ruler_entity_id(nlp, patterns):
    ruler = EntityRuler(nlp, patterns=patterns, overwrite_ents=True)
    nlp.add_pipe(ruler)
    doc = nlp("Apple is a technology company")
    assert len(doc.ents) == 1
    assert doc.ents[0].label_ == "TECH_ORG"
    assert doc.ents[0].ent_id_ == "a1"


def test_entity_ruler_cfg_ent_id_sep(nlp, patterns):
    ruler = EntityRuler(nlp, patterns=patterns, overwrite_ents=True, ent_id_sep="**")
    assert "TECH_ORG**a1" in ruler.phrase_patterns
    nlp.add_pipe(ruler)
    doc = nlp("Apple is a technology company")
    assert len(doc.ents) == 1
    assert doc.ents[0].label_ == "TECH_ORG"
    assert doc.ents[0].ent_id_ == "a1"


def test_entity_ruler_serialize_bytes(nlp, patterns):
    ruler = EntityRuler(nlp, patterns=patterns)
    assert len(ruler) == len(patterns)
    assert len(ruler.labels) == 4
    ruler_bytes = ruler.to_bytes()
    new_ruler = EntityRuler(nlp)
    assert len(new_ruler) == 0
    assert len(new_ruler.labels) == 0
    new_ruler = new_ruler.from_bytes(ruler_bytes)
    assert len(new_ruler) == len(patterns)
    assert len(new_ruler.labels) == 4
    assert len(new_ruler.patterns) == len(ruler.patterns)
    for pattern in ruler.patterns:
        assert pattern in new_ruler.patterns
    assert sorted(new_ruler.labels) == sorted(ruler.labels)


def test_entity_ruler_serialize_phrase_matcher_attr_bytes(nlp, patterns):
    ruler = EntityRuler(nlp, phrase_matcher_attr="LOWER", patterns=patterns)
    assert len(ruler) == len(patterns)
    assert len(ruler.labels) == 4
    ruler_bytes = ruler.to_bytes()
    new_ruler = EntityRuler(nlp)
    assert len(new_ruler) == 0
    assert len(new_ruler.labels) == 0
    assert new_ruler.phrase_matcher_attr is None
    new_ruler = new_ruler.from_bytes(ruler_bytes)
    assert len(new_ruler) == len(patterns)
    assert len(new_ruler.labels) == 4
    assert new_ruler.phrase_matcher_attr == "LOWER"


def test_entity_ruler_validate(nlp):
    ruler = EntityRuler(nlp)
    validated_ruler = EntityRuler(nlp, validate=True)

    valid_pattern = {"label": "HELLO", "pattern": [{"LOWER": "HELLO"}]}
    invalid_pattern = {"label": "HELLO", "pattern": [{"ASDF": "HELLO"}]}

    # invalid pattern raises error without validate
    with pytest.raises(ValueError):
        ruler.add_patterns([invalid_pattern])

    # valid pattern is added without errors with validate
    validated_ruler.add_patterns([valid_pattern])

    # invalid pattern raises error with validate
    with pytest.raises(MatchPatternError):
        validated_ruler.add_patterns([invalid_pattern])


def test_entity_ruler_properties(nlp, patterns):
    ruler = EntityRuler(nlp, patterns=patterns, overwrite_ents=True)
    assert sorted(ruler.labels) == sorted(["HELLO", "BYE", "COMPLEX", "TECH_ORG"])
    assert sorted(ruler.ent_ids) == ["a1", "a2"]



# coding: utf8
from __future__ import unicode_literals


STOP_WORDS = set(
    """
a abbastanza abbia abbiamo abbiano abbiate accidenti ad adesso affinche agl
agli ahime ahimè ai al alcuna alcuni alcuno all alla alle allo allora altri
altrimenti altro altrove altrui anche ancora anni anno ansa anticipo assai
attesa attraverso avanti avemmo avendo avente aver avere averlo avesse
avessero avessi avessimo aveste avesti avete aveva avevamo avevano avevate
avevi avevo avrai avranno avrebbe avrebbero avrei avremmo avremo avreste
avresti avrete avrà avrò avuta avute avuti avuto

basta bene benissimo brava bravo

casa caso cento certa certe certi certo che chi chicchessia chiunque ci
ciascuna ciascuno cima cio cioe circa citta città co codesta codesti codesto
cogli coi col colei coll coloro colui come cominci comunque con concernente
conciliarsi conclusione consiglio contro cortesia cos cosa cosi così cui

da dagl dagli dai dal dall dalla dalle dallo dappertutto davanti degl degli
dei del dell della delle dello dentro detto deve di dice dietro dire
dirimpetto diventa diventare diventato dopo dov dove dovra dovrà dovunque due
dunque durante

ebbe ebbero ebbi ecc ecco ed effettivamente egli ella entrambi eppure era
erano eravamo eravate eri ero esempio esse essendo esser essere essi ex

fa faccia facciamo facciano facciate faccio facemmo facendo facesse facessero
facessi facessimo faceste facesti faceva facevamo facevano facevate facevi
facevo fai fanno farai faranno fare farebbe farebbero farei faremmo faremo
fareste faresti farete farà farò fatto favore fece fecero feci fin finalmente
finche fine fino forse forza fosse fossero fossi fossimo foste fosti fra
frattempo fu fui fummo fuori furono futuro generale

gia già giacche giorni giorno gli gliela gliele glieli glielo gliene governo
grande grazie gruppo

ha haha hai hanno ho

ieri il improvviso in inc infatti inoltre insieme intanto intorno invece io

la là lasciato lato lavoro le lei li lo lontano loro lui lungo luogo

ma macche magari maggior mai male malgrado malissimo mancanza marche me
medesimo mediante meglio meno mentre mesi mezzo mi mia mie miei mila miliardi
milioni minimi ministro mio modo molti moltissimo molto momento mondo mosto

nazionale ne negl negli nei nel nell nella nelle nello nemmeno neppure nessun
nessuna nessuno niente no noi non nondimeno nonostante nonsia nostra nostre
nostri nostro novanta nove nulla nuovo

od oggi ogni ognuna ognuno oltre oppure ora ore osi ossia ottanta otto

paese parecchi parecchie parecchio parte partendo peccato peggio per perche
perché percio perciò perfino pero persino persone però piedi pieno piglia piu
piuttosto più po pochissimo poco poi poiche possa possedere posteriore posto
potrebbe preferibilmente presa press prima primo principalmente probabilmente
proprio puo può pure purtroppo

qualche qualcosa qualcuna qualcuno quale quali qualunque quando quanta quante
quanti quanto quantunque quasi quattro quel quella quelle quelli quello quest
questa queste questi questo qui quindi

realmente recente recentemente registrazione relativo riecco salvo

sara sarà sarai saranno sarebbe sarebbero sarei saremmo saremo sareste
saresti sarete saro sarò scola scopo scorso se secondo seguente seguito sei
sembra sembrare sembrato sembri sempre senza sette si sia siamo siano siate
siete sig solito solo soltanto sono sopra sotto spesso srl sta stai stando
stanno starai staranno starebbe starebbero starei staremmo staremo stareste
staresti starete starà starò stata state stati stato stava stavamo stavano
stavate stavi stavo stemmo stessa stesse stessero stessi stessimo stesso
steste stesti stette stettero stetti stia stiamo stiano stiate sto su sua
subito successivamente successivo sue sugl sugli sui sul sull sulla sulle
sullo suo suoi

tale tali talvolta tanto te tempo ti titolo torino tra tranne tre trenta
troppo trovato tu tua tue tuo tuoi tutta tuttavia tutte tutti tutto

uguali ulteriore ultimo un una uno uomo

va vale vari varia varie vario verso vi via vicino visto vita voi volta volte
vostra vostre vostri vostro
""".split()
)



# coding: utf8
from __future__ import unicode_literals
from thinc.v2v import Model, Maxout
from thinc.i2v import HashEmbed, StaticVectors
from thinc.t2t import ExtractWindow
from thinc.misc import Residual
from thinc.misc import LayerNorm as LN
from thinc.misc import FeatureExtracter
from thinc.api import layerize, chain, clone, concatenate, with_flatten
from thinc.api import uniqued, wrap, noop

from ..attrs import ID, ORTH, NORM, PREFIX, SUFFIX, SHAPE


def Tok2Vec(width, embed_size, **kwargs):
    # Circular imports :(
    from .._ml import CharacterEmbed
    from .._ml import PyTorchBiLSTM

    pretrained_vectors = kwargs.get("pretrained_vectors", None)
    cnn_maxout_pieces = kwargs.get("cnn_maxout_pieces", 3)
    subword_features = kwargs.get("subword_features", True)
    char_embed = kwargs.get("char_embed", False)
    if char_embed:
        subword_features = False
    conv_depth = kwargs.get("conv_depth", 4)
    bilstm_depth = kwargs.get("bilstm_depth", 0)
    cols = [ID, NORM, PREFIX, SUFFIX, SHAPE, ORTH]
    with Model.define_operators({">>": chain, "|": concatenate, "**": clone}):
        norm = HashEmbed(width, embed_size, column=cols.index(NORM), name="embed_norm")
        if subword_features:
            prefix = HashEmbed(
                width, embed_size // 2, column=cols.index(PREFIX), name="embed_prefix"
            )
            suffix = HashEmbed(
                width, embed_size // 2, column=cols.index(SUFFIX), name="embed_suffix"
            )
            shape = HashEmbed(
                width, embed_size // 2, column=cols.index(SHAPE), name="embed_shape"
            )
        else:
            prefix, suffix, shape = (None, None, None)
        if pretrained_vectors is not None:
            glove = StaticVectors(pretrained_vectors, width, column=cols.index(ID))

            if subword_features:
                embed = uniqued(
                    (glove | norm | prefix | suffix | shape)
                    >> LN(Maxout(width, width * 5, pieces=3)),
                    column=cols.index(ORTH),
                )
            else:
                embed = uniqued(
                    (glove | norm) >> LN(Maxout(width, width * 2, pieces=3)),
                    column=cols.index(ORTH),
                )
        elif subword_features:
            embed = uniqued(
                (norm | prefix | suffix | shape)
                >> LN(Maxout(width, width * 4, pieces=3)),
                column=cols.index(ORTH),
            )
        elif char_embed:
            embed = concatenate_lists(
                CharacterEmbed(nM=64, nC=8),
                FeatureExtracter(cols) >> with_flatten(norm),
            )
            reduce_dimensions = LN(
                Maxout(width, 64 * 8 + width, pieces=cnn_maxout_pieces)
            )
        else:
            embed = norm

        convolution = Residual(
            ExtractWindow(nW=1)
            >> LN(Maxout(width, width * 3, pieces=cnn_maxout_pieces))
        )
        if char_embed:
            tok2vec = embed >> with_flatten(
                reduce_dimensions >> convolution ** conv_depth, pad=conv_depth
            )
        else:
            tok2vec = FeatureExtracter(cols) >> with_flatten(
                embed >> convolution ** conv_depth, pad=conv_depth
            )

        if bilstm_depth >= 1:
            tok2vec = tok2vec >> PyTorchBiLSTM(width, width, bilstm_depth)
        # Work around thinc API limitations :(. TODO: Revise in Thinc 7
        tok2vec.nO = width
        tok2vec.embed = embed
    return tok2vec


@layerize
def flatten(seqs, drop=0.0):
    ops = Model.ops
    lengths = ops.asarray([len(seq) for seq in seqs], dtype="i")

    def finish_update(d_X, sgd=None):
        return ops.unflatten(d_X, lengths, pad=0)

    X = ops.flatten(seqs, pad=0)
    return X, finish_update


def concatenate_lists(*layers, **kwargs):  # pragma: no cover
    """Compose two or more models `f`, `g`, etc, such that their outputs are
    concatenated, i.e. `concatenate(f, g)(x)` computes `hstack(f(x), g(x))`
    """
    if not layers:
        return noop()
    drop_factor = kwargs.get("drop_factor", 1.0)
    ops = layers[0].ops
    layers = [chain(layer, flatten) for layer in layers]
    concat = concatenate(*layers)

    def concatenate_lists_fwd(Xs, drop=0.0):
        if drop is not None:
            drop *= drop_factor
        lengths = ops.asarray([len(X) for X in Xs], dtype="i")
        flat_y, bp_flat_y = concat.begin_update(Xs, drop=drop)
        ys = ops.unflatten(flat_y, lengths)

        def concatenate_lists_bwd(d_ys, sgd=None):
            return bp_flat_y(ops.flatten(d_ys), sgd=sgd)

        return ys, concatenate_lists_bwd

    model = wrap(concatenate_lists_fwd, concat)
    return model



# coding: utf8
from __future__ import unicode_literals


# Source: https://github.com/sanjaalcorps/NepaliStopWords/blob/master/NepaliStopWords.txt

STOP_WORDS = set(
    """
अक्सर
अगाडि
अगाडी
अघि
अझै
अठार
अथवा
अनि
अनुसार
अन्तर्गत
अन्य
अन्यत्र
अन्यथा
अब
अरु
अरुलाई
अरू
अर्को
अर्थात
अर्थात्
अलग
अलि
अवस्था
अहिले
आए
आएका
आएको
आज
आजको
आठ
आत्म
आदि
आदिलाई
आफनो
आफू
आफूलाई
आफै
आफैँ
आफ्नै
आफ्नो
आयो
उ
उक्त
उदाहरण
उनको
उनलाई
उनले
उनि
उनी
उनीहरुको
उन्नाइस
उप
उसको
उसलाई
उसले
उहालाई
ऊ
एउटा
एउटै
एक
एकदम
एघार
ओठ
औ
औं
कता
कति
कतै
कम
कमसेकम
कसरि
कसरी
कसै
कसैको
कसैलाई
कसैले
कसैसँग
कस्तो
कहाँबाट
कहिलेकाहीं
का
काम
कारण
कि
किन
किनभने
कुन
कुनै
कुन्नी
कुरा
कृपया
के
केहि
केही
को
कोहि
कोहिपनि
कोही
कोहीपनि
क्रमशः
गए
गएको
गएर
गयौ
गरि
गरी
गरे
गरेका
गरेको
गरेर
गरौं
गर्छ
गर्छन्
गर्छु
गर्दा
गर्दै
गर्न
गर्नु
गर्नुपर्छ
गर्ने
गैर
घर
चार
चाले
चाहनुहुन्छ
चाहन्छु
चाहिं
चाहिए
चाहिंले
चाहीं
चाहेको
चाहेर
चोटी
चौथो
चौध
छ
छन
छन्
छु
छू
छैन
छैनन्
छौ
छौं
जता
जताततै
जना
जनाको
जनालाई
जनाले
जब
जबकि
जबकी
जसको
जसबाट
जसमा
जसरी
जसलाई
जसले
जस्ता
जस्तै
जस्तो
जस्तोसुकै
जहाँ
जान
जाने
जाहिर
जुन
जुनै
जे
जो
जोपनि
जोपनी
झैं
ठाउँमा
ठीक
ठूलो
त
तता
तत्काल
तथा
तथापि
तथापी
तदनुसार
तपाइ
तपाई
तपाईको
तब
तर
तर्फ
तल
तसरी
तापनि
तापनी
तिन
तिनि
तिनिहरुलाई
तिनी
तिनीहरु
तिनीहरुको
तिनीहरू
तिनीहरूको
तिनै
तिमी
तिर
तिरको
ती
तीन
तुरन्त
तुरुन्त
तुरुन्तै
तेश्रो
तेस्कारण
तेस्रो
तेह्र
तैपनि
तैपनी
त्यत्तिकै
त्यत्तिकैमा
त्यस
त्यसकारण
त्यसको
त्यसले
त्यसैले
त्यसो
त्यस्तै
त्यस्तो
त्यहाँ
त्यहिँ
त्यही
त्यहीँ
त्यहीं
त्यो
त्सपछि
त्सैले
थप
थरि
थरी
थाहा
थिए
थिएँ
थिएन
थियो
दर्ता
दश
दिए
दिएको
दिन
दिनुभएको
दिनुहुन्छ
दुइ
दुइवटा
दुई
देखि
देखिन्छ
देखियो
देखे
देखेको
देखेर
दोश्री
दोश्रो
दोस्रो
द्वारा
धन्न
धेरै
धौ
न
नगर्नु
नगर्नू
नजिकै
नत्र
नत्रभने
नभई
नभएको
नभनेर
नयाँ
नि
निकै
निम्ति
निम्न
निम्नानुसार
निर्दिष्ट
नै
नौ
पक्का
पक्कै
पछाडि
पछाडी
पछि
पछिल्लो
पछी
पटक
पनि
पन्ध्र
पर्छ
पर्थ्यो
पर्दैन
पर्ने
पर्नेमा
पर्याप्त
पहिले
पहिलो
पहिल्यै
पाँच
पांच
पाचौँ
पाँचौं
पिच्छे
पूर्व
पो
प्रति
प्रतेक
प्रत्यक
प्राय
प्लस
फरक
फेरि
फेरी
बढी
बताए
बने
बरु
बाट
बारे
बाहिर
बाहेक
बाह्र
बिच
बिचमा
बिरुद्ध
बिशेष
बिस
बीच
बीचमा
बीस
भए
भएँ
भएका
भएकालाई
भएको
भएन
भएर
भन
भने
भनेको
भनेर
भन्
भन्छन्
भन्छु
भन्दा
भन्दै
भन्नुभयो
भन्ने
भन्या
भयेन
भयो
भर
भरि
भरी
भा
भित्र
भित्री
भीत्र
म
मध्य
मध्ये
मलाई
मा
मात्र
मात्रै
माथि
माथी
मुख्य
मुनि
मुन्तिर
मेरो
मैले
यति
यथोचित
यदि
यद्ध्यपि
यद्यपि
यस
यसका
यसको
यसपछि
यसबाहेक
यसमा
यसरी
यसले
यसो
यस्तै
यस्तो
यहाँ
यहाँसम्म
यही
या
यी
यो
र
रही
रहेका
रहेको
रहेछ
राखे
राख्छ
राम्रो
रुपमा
रूप
रे
लगभग
लगायत
लाई
लाख
लागि
लागेको
ले
वटा
वरीपरी
वा
वाट
वापत
वास्तवमा
शायद
सक्छ
सक्ने
सँग
संग
सँगको
सँगसँगै
सँगै
संगै
सङ्ग
सङ्गको
सट्टा
सत्र
सधै
सबै
सबैको
सबैलाई
समय
समेत
सम्भव
सम्म
सय
सरह
सहित
सहितै
सही
साँच्चै
सात
साथ
साथै
सायद
सारा
सुनेको
सुनेर
सुरु
सुरुको
सुरुमै
सो
सोचेको
सोचेर
सोही
सोह्र
स्थित
स्पष्ट
हजार
हरे
हरेक
हामी
हामीले
हाम्रा
हाम्रो
हुँदैन
हुन
हुनत
हुनु
हुने
हुनेछ
हुन्
हुन्छ
हुन्थ्यो
हैन
हो
होइन
होकि
होला
""".split()
)



# coding: utf8
from __future__ import unicode_literals


"""
Example sentences to test spaCy and its language models.

>>> from spacy.lang.da.examples import sentences
>>> docs = nlp.pipe(sentences)
"""


sentences = [
    "اردو ہے جس کا نام ہم جانتے ہیں داغ",
    "سارے جہاں میں دھوم ہماری زباں کی ہے",
]



# coding: utf8
from __future__ import unicode_literals

import pytest
from spacy.tokens import Span
from spacy.language import Language
from spacy.pipeline import EntityRuler
from spacy import load
import srsly

from ..util import make_tempdir


@pytest.fixture
def patterns():
    return [
        {"label": "HELLO", "pattern": "hello world"},
        {"label": "BYE", "pattern": [{"LOWER": "bye"}, {"LOWER": "bye"}]},
        {"label": "HELLO", "pattern": [{"ORTH": "HELLO"}]},
        {"label": "COMPLEX", "pattern": [{"ORTH": "foo", "OP": "*"}]},
        {"label": "TECH_ORG", "pattern": "Apple", "id": "a1"},
    ]


@pytest.fixture
def add_ent():
    def add_ent_component(doc):
        doc.ents = [Span(doc, 0, 3, label=doc.vocab.strings["ORG"])]
        return doc

    return add_ent_component


def test_entity_ruler_existing_overwrite_serialize_bytes(patterns, en_vocab):
    nlp = Language(vocab=en_vocab)
    ruler = EntityRuler(nlp, patterns=patterns, overwrite_ents=True)
    ruler_bytes = ruler.to_bytes()
    assert len(ruler) == len(patterns)
    assert len(ruler.labels) == 4
    assert ruler.overwrite
    new_ruler = EntityRuler(nlp)
    new_ruler = new_ruler.from_bytes(ruler_bytes)
    assert len(new_ruler) == len(ruler)
    assert len(new_ruler.labels) == 4
    assert new_ruler.overwrite == ruler.overwrite
    assert new_ruler.ent_id_sep == ruler.ent_id_sep


def test_entity_ruler_existing_bytes_old_format_safe(patterns, en_vocab):
    nlp = Language(vocab=en_vocab)
    ruler = EntityRuler(nlp, patterns=patterns, overwrite_ents=True)
    bytes_old_style = srsly.msgpack_dumps(ruler.patterns)
    new_ruler = EntityRuler(nlp)
    new_ruler = new_ruler.from_bytes(bytes_old_style)
    assert len(new_ruler) == len(ruler)
    for pattern in ruler.patterns:
        assert pattern in new_ruler.patterns
    assert new_ruler.overwrite is not ruler.overwrite


def test_entity_ruler_from_disk_old_format_safe(patterns, en_vocab):
    nlp = Language(vocab=en_vocab)
    ruler = EntityRuler(nlp, patterns=patterns, overwrite_ents=True)
    with make_tempdir() as tmpdir:
        out_file = tmpdir / "entity_ruler"
        srsly.write_jsonl(out_file.with_suffix(".jsonl"), ruler.patterns)
        new_ruler = EntityRuler(nlp).from_disk(out_file)
        for pattern in ruler.patterns:
            assert pattern in new_ruler.patterns
        assert len(new_ruler) == len(ruler)
        assert new_ruler.overwrite is not ruler.overwrite


def test_entity_ruler_in_pipeline_from_issue(patterns, en_vocab):
    nlp = Language(vocab=en_vocab)
    ruler = EntityRuler(nlp, overwrite_ents=True)

    ruler.add_patterns([{"label": "ORG", "pattern": "Apple"}])
    nlp.add_pipe(ruler)
    with make_tempdir() as tmpdir:
        nlp.to_disk(tmpdir)
        ruler = nlp.get_pipe("entity_ruler")
        assert ruler.patterns == [{"label": "ORG", "pattern": "Apple"}]
        assert ruler.overwrite is True
        nlp2 = load(tmpdir)
        new_ruler = nlp2.get_pipe("entity_ruler")
        assert new_ruler.patterns == [{"label": "ORG", "pattern": "Apple"}]
        assert new_ruler.overwrite is True



# coding: utf8
from __future__ import unicode_literals

# This list was created by taking the top 2000 words from a Wikipedia dump and
# filtering out everything that wasn't hiragana. ー (one) was also added.
# Considered keeping some non-hiragana words but too many place names were
# present.
STOP_WORDS = set(
    """
あ あっ あまり あり ある あるいは あれ
い いい いう いく いずれ いっ いつ いる いわ
うち
え
お おい おけ および おら おり
か かけ かつ かつて かなり から が
き きっかけ
くる くん
こ こう ここ こと この これ ご ごと
さ さらに さん
し しか しかし しまう しまっ しよう
す すぐ すべて する ず
せ せい せる
そう そこ そして その それ それぞれ
た たい ただし たち ため たら たり だ だけ だっ
ち ちゃん
つ つい つけ つつ
て で でき できる です
と とき ところ とっ とも どう
な ない なお なかっ ながら なく なけれ なし なっ など なら なり なる
に にて
ぬ
ね
の のち のみ
は はじめ ば
ひと
ぶり
へ べき
ほか ほとんど ほど ほぼ
ま ます また まで まま
み
も もう もっ もと もの
や やっ
よ よう よく よっ より よる よれ
ら らしい られ られる
る
れ れる
を
ん
一
""".split()
)



# coding: utf-8
from __future__ import unicode_literals


# NB: This schema describes the new format of the training data, see #2928
TRAINING_SCHEMA = {
    "$schema": "http://json-schema.org/draft-06/schema",
    "title": "Training data for spaCy models",
    "type": "array",
    "items": {
        "type": "object",
        "properties": {
            "text": {
                "title": "The text of the training example",
                "type": "string",
                "minLength": 1,
            },
            "ents": {
                "title": "Named entity spans in the text",
                "type": "array",
                "items": {
                    "type": "object",
                    "properties": {
                        "start": {
                            "title": "Start character offset of the span",
                            "type": "integer",
                            "minimum": 0,
                        },
                        "end": {
                            "title": "End character offset of the span",
                            "type": "integer",
                            "minimum": 0,
                        },
                        "label": {
                            "title": "Entity label",
                            "type": "string",
                            "minLength": 1,
                            "pattern": "^[A-Z0-9]*$",
                        },
                    },
                    "required": ["start", "end", "label"],
                },
            },
            "sents": {
                "title": "Sentence spans in the text",
                "type": "array",
                "items": {
                    "type": "object",
                    "properties": {
                        "start": {
                            "title": "Start character offset of the span",
                            "type": "integer",
                            "minimum": 0,
                        },
                        "end": {
                            "title": "End character offset of the span",
                            "type": "integer",
                            "minimum": 0,
                        },
                    },
                    "required": ["start", "end"],
                },
            },
            "cats": {
                "title": "Text categories for the text classifier",
                "type": "object",
                "patternProperties": {
                    "*": {
                        "title": "A text category",
                        "oneOf": [
                            {"type": "boolean"},
                            {"type": "number", "minimum": 0},
                        ],
                    }
                },
                "propertyNames": {"pattern": "^[A-Z0-9]*$", "minLength": 1},
            },
            "tokens": {
                "title": "The tokens in the text",
                "type": "array",
                "items": {
                    "type": "object",
                    "minProperties": 1,
                    "properties": {
                        "id": {
                            "title": "Token ID, usually token index",
                            "type": "integer",
                            "minimum": 0,
                        },
                        "start": {
                            "title": "Start character offset of the token",
                            "type": "integer",
                            "minimum": 0,
                        },
                        "end": {
                            "title": "End character offset of the token",
                            "type": "integer",
                            "minimum": 0,
                        },
                        "pos": {
                            "title": "Coarse-grained part-of-speech tag",
                            "type": "string",
                            "minLength": 1,
                        },
                        "tag": {
                            "title": "Fine-grained part-of-speech tag",
                            "type": "string",
                            "minLength": 1,
                        },
                        "dep": {
                            "title": "Dependency label",
                            "type": "string",
                            "minLength": 1,
                        },
                        "head": {
                            "title": "Index of the token's head",
                            "type": "integer",
                            "minimum": 0,
                        },
                    },
                    "required": ["start", "end"],
                },
            },
            "_": {"title": "Custom user space", "type": "object"},
        },
        "required": ["text"],
    },
}

META_SCHEMA = {
    "$schema": "http://json-schema.org/draft-06/schema",
    "type": "object",
    "properties": {
        "lang": {
            "title": "Two-letter language code, e.g. 'en'",
            "type": "string",
            "minLength": 2,
            "maxLength": 2,
            "pattern": "^[a-z]*$",
        },
        "name": {
            "title": "Model name",
            "type": "string",
            "minLength": 1,
            "pattern": "^[a-z_]*$",
        },
        "version": {
            "title": "Model version",
            "type": "string",
            "minLength": 1,
            "pattern": "^[0-9a-z.-]*$",
        },
        "spacy_version": {
            "title": "Compatible spaCy version identifier",
            "type": "string",
            "minLength": 1,
            "pattern": "^[0-9a-z.-><=]*$",
        },
        "parent_package": {
            "title": "Name of parent spaCy package, e.g. spacy or spacy-nightly",
            "type": "string",
            "minLength": 1,
            "default": "spacy",
        },
        "pipeline": {
            "title": "Names of pipeline components",
            "type": "array",
            "items": {"type": "string", "minLength": 1},
        },
        "description": {"title": "Model description", "type": "string"},
        "license": {"title": "Model license", "type": "string"},
        "author": {"title": "Model author name", "type": "string"},
        "email": {"title": "Model author email", "type": "string", "format": "email"},
        "url": {"title": "Model author URL", "type": "string", "format": "uri"},
        "sources": {
            "title": "Training data sources",
            "type": "array",
            "items": {"type": "string"},
        },
        "vectors": {
            "title": "Included word vectors",
            "type": "object",
            "properties": {
                "keys": {
                    "title": "Number of unique keys",
                    "type": "integer",
                    "minimum": 0,
                },
                "vectors": {
                    "title": "Number of unique vectors",
                    "type": "integer",
                    "minimum": 0,
                },
                "width": {
                    "title": "Number of dimensions",
                    "type": "integer",
                    "minimum": 0,
                },
            },
        },
        "accuracy": {
            "title": "Accuracy numbers",
            "type": "object",
            "patternProperties": {"*": {"type": "number", "minimum": 0.0}},
        },
        "speed": {
            "title": "Speed evaluation numbers",
            "type": "object",
            "patternProperties": {
                "*": {
                    "oneOf": [
                        {"type": "number", "minimum": 0.0},
                        {"type": "integer", "minimum": 0},
                    ]
                }
            },
        },
    },
    "required": ["lang", "name", "version"],
}



# coding: utf8
from __future__ import unicode_literals

import plac
import platform
from pathlib import Path
from wasabi import msg
import srsly

from ..compat import path2str, basestring_, unicode_
from .. import util
from .. import about


@plac.annotations(
    model=("Optional shortcut link of model", "positional", None, str),
    markdown=("Generate Markdown for GitHub issues", "flag", "md", str),
    silent=("Don't print anything (just return)", "flag", "s"),
)
def info(model=None, markdown=False, silent=False):
    """
    Print info about spaCy installation. If a model shortcut link is
    speficied as an argument, print model information. Flag --markdown
    prints details in Markdown for easy copy-pasting to GitHub issues.
    """
    if model:
        if util.is_package(model):
            model_path = util.get_package_path(model)
        else:
            model_path = util.get_data_path() / model
        meta_path = model_path / "meta.json"
        if not meta_path.is_file():
            msg.fail("Can't find model meta.json", meta_path, exits=1)
        meta = srsly.read_json(meta_path)
        if model_path.resolve() != model_path:
            meta["link"] = path2str(model_path)
            meta["source"] = path2str(model_path.resolve())
        else:
            meta["source"] = path2str(model_path)
        if not silent:
            title = "Info about model '{}'".format(model)
            model_meta = {
                k: v for k, v in meta.items() if k not in ("accuracy", "speed")
            }
            if markdown:
                print_markdown(model_meta, title=title)
            else:
                msg.table(model_meta, title=title)
        return meta
    data = {
        "spaCy version": about.__version__,
        "Location": path2str(Path(__file__).parent.parent),
        "Platform": platform.platform(),
        "Python version": platform.python_version(),
        "Models": list_models(),
    }
    if not silent:
        title = "Info about spaCy"
        if markdown:
            print_markdown(data, title=title)
        else:
            msg.table(data, title=title)
    return data


def list_models():
    def exclude_dir(dir_name):
        # exclude common cache directories and hidden directories
        exclude = ("cache", "pycache", "__pycache__")
        return dir_name in exclude or dir_name.startswith(".")

    data_path = util.get_data_path()
    if data_path:
        models = [f.parts[-1] for f in data_path.iterdir() if f.is_dir()]
        return ", ".join([m for m in models if not exclude_dir(m)])
    return "-"


def print_markdown(data, title=None):
    """Print data in GitHub-flavoured Markdown format for issues etc.

    data (dict or list of tuples): Label/value pairs.
    title (unicode or None): Title, will be rendered as headline 2.
    """
    markdown = []
    for key, value in data.items():
        if isinstance(value, basestring_) and Path(value).exists():
            continue
        markdown.append("* **{}:** {}".format(key, unicode_(value)))
    if title:
        print("\n## {}".format(title))
    print("\n{}\n".format("\n".join(markdown)))



# coding: utf8
from __future__ import unicode_literals

from spacy.lang.en import English
from spacy.pipeline import EntityRuler


def test_issue4267():
    """ Test that running an entity_ruler after ner gives consistent results"""
    nlp = English()
    ner = nlp.create_pipe("ner")
    ner.add_label("PEOPLE")
    nlp.add_pipe(ner)
    nlp.begin_training()

    assert "ner" in nlp.pipe_names

    # assert that we have correct IOB annotations
    doc1 = nlp("hi")
    assert doc1.is_nered
    for token in doc1:
        assert token.ent_iob == 2

    # add entity ruler and run again
    ruler = EntityRuler(nlp)
    patterns = [{"label": "SOFTWARE", "pattern": "spacy"}]

    ruler.add_patterns(patterns)
    nlp.add_pipe(ruler)
    assert "entity_ruler" in nlp.pipe_names
    assert "ner" in nlp.pipe_names

    # assert that we still have correct IOB annotations
    doc2 = nlp("hi")
    assert doc2.is_nered
    for token in doc2:
        assert token.ent_iob == 2



# coding: utf8
from __future__ import unicode_literals

from spacy.lang.hi import Hindi


def test_issue3625():
    """Test that default punctuation rules applies to hindi unicode characters"""
    nlp = Hindi()
    doc = nlp("hi. how हुए. होटल, होटल")
    expected = ["hi", ".", "how", "हुए", ".", "होटल", ",", "होटल"]
    assert [token.text for token in doc] == expected



# coding: utf8
from __future__ import unicode_literals

from spacy.displacy import parse_deps
from spacy.tokens import Doc


def test_issue3882(en_vocab):
    """Test that displaCy doesn't serialize the doc.user_data when making a
    copy of the Doc.
    """
    doc = Doc(en_vocab, words=["Hello", "world"])
    doc.is_parsed = True
    doc.user_data["test"] = set()
    parse_deps(doc)



# coding: utf8
from __future__ import unicode_literals

import plac
from pathlib import Path
from wasabi import msg

from ..compat import symlink_to, path2str
from .. import util


@plac.annotations(
    origin=("package name or local path to model", "positional", None, str),
    link_name=("name of shortuct link to create", "positional", None, str),
    force=("force overwriting of existing link", "flag", "f", bool),
)
def link(origin, link_name, force=False, model_path=None):
    """
    Create a symlink for models within the spacy/data directory. Accepts
    either the name of a pip package, or the local path to the model data
    directory. Linking models allows loading them via spacy.load(link_name).
    """
    if util.is_package(origin):
        model_path = util.get_package_path(origin)
    else:
        model_path = Path(origin) if model_path is None else Path(model_path)
    if not model_path.exists():
        msg.fail(
            "Can't locate model data",
            "The data should be located in {}".format(path2str(model_path)),
            exits=1,
        )
    data_path = util.get_data_path()
    if not data_path or not data_path.exists():
        spacy_loc = Path(__file__).parent.parent
        msg.fail(
            "Can't find the spaCy data path to create model symlink",
            "Make sure a directory `/data` exists within your spaCy "
            "installation and try again. The data directory should be located "
            "here:".format(path=spacy_loc),
            exits=1,
        )
    link_path = util.get_data_path() / link_name
    if link_path.is_symlink() and not force:
        msg.fail(
            "Link '{}' already exists".format(link_name),
            "To overwrite an existing link, use the --force flag",
            exits=1,
        )
    elif link_path.is_symlink():  # does a symlink exist?
        # NB: It's important to check for is_symlink here and not for exists,
        # because invalid/outdated symlinks would return False otherwise.
        link_path.unlink()
    elif link_path.exists():  # does it exist otherwise?
        # NB: Check this last because valid symlinks also "exist".
        msg.fail(
            "Can't overwrite symlink '{}'".format(link_name),
            "This can happen if your data directory contains a directory or "
            "file of the same name.",
            exits=1,
        )
    details = "%s --> %s" % (path2str(model_path), path2str(link_path))
    try:
        symlink_to(link_path, model_path)
    except:  # noqa: E722
        # This is quite dirty, but just making sure other errors are caught.
        msg.fail(
            "Couldn't link model to '{}'".format(link_name),
            "Creating a symlink in spacy/data failed. Make sure you have the "
            "required permissions and try re-running the command as admin, or "
            "use a virtualenv. You can still import the model as a module and "
            "call its load() method, or create the symlink manually.",
        )
        msg.text(details)
        raise
    msg.good("Linking successful", details)
    msg.text("You can now load the model via spacy.load('{}')".format(link_name))



# coding: utf8
from __future__ import unicode_literals

import pytest
from spacy.language import Language
from spacy.tokens import Span

from ..util import get_doc


@pytest.fixture
def doc(en_tokenizer):
    text = "I like New York in Autumn."
    heads = [1, 0, 1, -2, -3, -1, -5]
    tags = ["PRP", "IN", "NNP", "NNP", "IN", "NNP", "."]
    pos = ["PRON", "VERB", "PROPN", "PROPN", "ADP", "PROPN", "PUNCT"]
    deps = ["ROOT", "prep", "compound", "pobj", "prep", "pobj", "punct"]
    tokens = en_tokenizer(text)
    doc = get_doc(
        tokens.vocab,
        words=[t.text for t in tokens],
        heads=heads,
        tags=tags,
        pos=pos,
        deps=deps,
    )
    doc.ents = [Span(doc, 2, 4, doc.vocab.strings["GPE"])]
    doc.is_parsed = True
    doc.is_tagged = True
    return doc


def test_factories_merge_noun_chunks(doc):
    assert len(doc) == 7
    nlp = Language()
    merge_noun_chunks = nlp.create_pipe("merge_noun_chunks")
    merge_noun_chunks(doc)
    assert len(doc) == 6
    assert doc[2].text == "New York"


def test_factories_merge_ents(doc):
    assert len(doc) == 7
    assert len(list(doc.ents)) == 1
    nlp = Language()
    merge_entities = nlp.create_pipe("merge_entities")
    merge_entities(doc)
    assert len(doc) == 6
    assert len(list(doc.ents)) == 1
    assert doc[2].text == "New York"



# coding: utf-8
from __future__ import unicode_literals

import pytest


@pytest.mark.parametrize("text", ["(under)"])
def test_tokenizer_splits_no_special(sv_tokenizer, text):
    tokens = sv_tokenizer(text)
    assert len(tokens) == 3


@pytest.mark.parametrize("text", ["gitta'r", "Björn's", "Lars'"])
def test_tokenizer_handles_no_punct(sv_tokenizer, text):
    tokens = sv_tokenizer(text)
    assert len(tokens) == 1


@pytest.mark.parametrize("text", ["svart.Gul", "Hej.Världen"])
def test_tokenizer_splits_period_infix(sv_tokenizer, text):
    tokens = sv_tokenizer(text)
    assert len(tokens) == 3


@pytest.mark.parametrize("text", ["Hej,Världen", "en,två"])
def test_tokenizer_splits_comma_infix(sv_tokenizer, text):
    tokens = sv_tokenizer(text)
    assert len(tokens) == 3
    assert tokens[0].text == text.split(",")[0]
    assert tokens[1].text == ","
    assert tokens[2].text == text.split(",")[1]


@pytest.mark.parametrize("text", ["svart...Gul", "svart...gul"])
def test_tokenizer_splits_ellipsis_infix(sv_tokenizer, text):
    tokens = sv_tokenizer(text)
    assert len(tokens) == 3



# coding: utf-8
from __future__ import unicode_literals

from ...util import get_doc


def test_en_tagger_load_morph_exc(en_tokenizer):
    text = "I like his style."
    tags = ["PRP", "VBP", "PRP$", "NN", "."]
    morph_exc = {"VBP": {"like": {"lemma": "luck"}}}
    en_tokenizer.vocab.morphology.load_morph_exceptions(morph_exc)
    tokens = en_tokenizer(text)
    doc = get_doc(tokens.vocab, words=[t.text for t in tokens], tags=tags)
    assert doc[1].tag_ == "VBP"
    assert doc[1].lemma_ == "luck"



# coding: utf8
from __future__ import unicode_literals

from ..char_classes import ALPHA, ALPHA_LOWER, ALPHA_UPPER, CONCAT_QUOTES, HYPHENS
from ..char_classes import LIST_ELLIPSES, LIST_ICONS

_hyphens_no_dash = HYPHENS.replace("-", "").strip("|").replace("||", "")
_infixes = (
    LIST_ELLIPSES
    + LIST_ICONS
    + [
        r"(?<=[{al}])\.(?=[{au}])".format(al=ALPHA_LOWER, au=ALPHA_UPPER),
        r"(?<=[{a}])[,!?/()]+(?=[{a}])".format(a=ALPHA),
        r"(?<=[{a}{q}])[:<>=](?=[{a}])".format(a=ALPHA, q=CONCAT_QUOTES),
        r"(?<=[{a}])--(?=[{a}])".format(a=ALPHA),
        r"(?<=[{a}]),(?=[{a}])".format(a=ALPHA),
        r"(?<=[{a}])([{q}\)\]\(\[])(?=[\-{a}])".format(a=ALPHA, q=CONCAT_QUOTES),
        r"(?<=[{a}])(?:{h})(?=[{a}])".format(a=ALPHA, h=_hyphens_no_dash),
        r"(?<=[0-9])-(?=[0-9])",
    ]
)

TOKENIZER_INFIXES = _infixes



from cymem.cymem cimport Pool
from preshed.maps cimport PreshMap, PreshMapArray
from libc.stdint cimport uint64_t
from murmurhash cimport mrmr

from .structs cimport TokenC, MorphAnalysisC
from .strings cimport StringStore
from .typedefs cimport hash_t, attr_t, flags_t
from .parts_of_speech cimport univ_pos_t

from . cimport symbols

cdef class Morphology:
    cdef readonly Pool mem
    cdef readonly StringStore strings
    cdef PreshMap tags # Keyed by hash, value is pointer to tag
 
    cdef public object lemmatizer
    cdef readonly object tag_map
    cdef readonly object tag_names
    cdef readonly object reverse_index
    cdef readonly object exc
    cdef readonly object _feat_map
    cdef readonly PreshMapArray _cache
    cdef readonly int n_tags

    cpdef update(self, hash_t morph, features)
    cdef hash_t insert(self, MorphAnalysisC tag) except 0
    
    cdef int assign_untagged(self, TokenC* token) except -1
    cdef int assign_tag(self, TokenC* token, tag) except -1
    cdef int assign_tag_id(self, TokenC* token, int tag_id) except -1

    cdef int _assign_tag_from_exceptions(self, TokenC* token, int tag_id) except -1


cdef int check_feature(const MorphAnalysisC* tag, attr_t feature) nogil
cdef attr_t get_field(const MorphAnalysisC* tag, int field) nogil
cdef list list_features(const MorphAnalysisC* tag)

cdef tag_to_json(const MorphAnalysisC* tag)
